import 'dart:collection';
import 'dart:convert';
import 'dart:io';

import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:path_provider/path_provider.dart';
import 'package:tencent_cloud_chat_sdk/enum/V2TimAdvancedMsgListener.dart';
import 'package:tencent_cloud_chat_sdk/enum/V2TimConversationListener.dart';
import 'package:tencent_cloud_chat_sdk/enum/V2TimFriendshipListener.dart';
import 'package:tencent_cloud_chat_sdk/enum/V2TimGroupListener.dart';
import 'package:tencent_cloud_chat_sdk/enum/V2TimSDKListener.dart';
import 'package:tencent_cloud_chat_sdk/enum/V2TimSignalingListener.dart';
import 'package:tencent_cloud_chat_sdk/enum/V2TimSimpleMsgListener.dart';
import 'package:tencent_cloud_chat_sdk/enum/callbacks.dart';
import 'package:tencent_cloud_chat_sdk/enum/get_group_message_read_member_list_filter.dart';
import 'package:tencent_cloud_chat_sdk/enum/history_message_get_type.dart';
import 'package:tencent_cloud_chat_sdk/enum/offlinePushInfo.dart';
import 'package:tencent_cloud_chat_sdk/enum/v2_tim_plugins.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_callback.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_conversation.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_conversation_filter.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_conversation_operation_result.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_conversation_result.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_friend_application_result.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_friend_check_result.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_friend_group.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_friend_info.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_friend_info_result.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_friend_operation_result.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_friend_search_param.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_group_application_result.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_group_info.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_group_info_result.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_group_member.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_group_member_full_info.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_group_member_info_result.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_group_member_operation_result.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_group_member_search_param.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_group_member_search_result.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_group_message_read_member_list.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_group_search_param.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_message.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_message_change_info.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_message_extension.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_message_extension_result.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_message_list_result.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_message_online_url.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_message_reaction_result.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_message_reaction_user_result.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_message_receipt.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_message_search_param.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_message_search_result.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_msg_create_info_result.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_receive_message_opt_info.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_signaling_info.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_topic_info.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_topic_info_result.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_topic_operation_result.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_user_full_info.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_user_status.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_value_callback.dart';
import 'package:tencent_cloud_chat_sdk/utils/utils.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_message_download_progress.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_message_reaction_change_info.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_user_info.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_friend_application.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_group_change_info.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_group_member_change_info.dart';
import 'package:tencent_cloud_chat_sdk/models/v2_tim_group_member_info.dart';
import 'package:tencent_cloud_chat_sdk/enum/listener_type.dart';
import 'tencent_cloud_chat_sdk_platform_interface.dart';

/// An implementation of [TencentCloudChatSdkPlatform] that uses method channels.
class MethodChannelTencentCloudChatSdk extends TencentCloudChatSdkPlatform {
  /// The method channel used to interact with the native platform.
  static const _channel = MethodChannel('tencent_cloud_chat_sdk');

  static final Map<String, V2TimSimpleMsgListener> simpleMessageListenerList = {};
  static final Map<String, V2TimSDKListener> initSDKListenerList = {};

  static final Map<String, V2TimGroupListener> groupListenerList = {};

  static final Map<String, V2TimConversationListener> conversationListenerList = {};

  static final Map<String, V2TimFriendshipListener> friendListenerList = {};

  static final Map<String, V2TimSignalingListener> signalingListenerList = {};
  static final Map<String, V2TimAdvancedMsgListener> advancedMsgListenerList = {};
  @override
  Future<String?> getPlatformVersion() async {
    final version = await _channel.invokeMethod<String>('getPlatformVersion', buildTimManagerParam({}));
    return version;
  }

  _catchListenerError(Function listener) {
    try {
      listener();
    } catch (err, errorStack) {
      debugPrint("$err $errorStack");
    }
  }

  @override
  void addNativeCallback() {
    _channel.setMethodCallHandler((call) async {
      try {
        if (call.method == ListenerType.simpleMsgListener) {
          Map<String, dynamic> data = formatJson(call.arguments);
          Map<String, dynamic> params = data['data'] ?? Map<String, dynamic>.from({});
          String type = data['type'];
          simpleMessageListenerList.forEach((key, value) {
            final msgID = params['msgID'];
            switch (type) {
              case 'onRecvC2CCustomMessage':
                // String msgID, V2TIMUserInfo sender, byte[] customData
                final sender = V2TimUserInfo.fromJson(params['sender']);
                _catchListenerError(() {
                  value.onRecvC2CCustomMessage(
                    msgID,
                    sender,
                    params['customData'],
                  );
                });
                break;
              case 'onRecvC2CTextMessage':
                // String msgID, V2TIMUserInfo sender, String text
                final sender = V2TimUserInfo.fromJson(params['sender']);
                _catchListenerError(() {
                  value.onRecvC2CTextMessage(
                    msgID,
                    sender,
                    params['text'],
                  );
                });
                break;
              case 'onRecvGroupCustomMessage':
                // String msgID, String groupID, V2TIMGroupMemberInfo sender, byte[] customData
                final groupSender = V2TimGroupMemberInfo.fromJson(params['sender']);
                _catchListenerError(() {
                  value.onRecvGroupCustomMessage(
                    msgID,
                    params['groupID'],
                    groupSender,
                    params['customData'],
                  );
                });
                break;
              case 'onRecvGroupTextMessage':
                // String msgID, String groupID, V2TIMGroupMemberInfo sender, String text
                final groupSender = V2TimGroupMemberInfo.fromJson(params['sender']);
                _catchListenerError(() {
                  value.onRecvGroupTextMessage(
                    params['msgID'],
                    params['groupID'],
                    groupSender,
                    params['text'],
                  );
                });
                break;
            }
          });
        } else if (call.method == ListenerType.initSDKListener) {
          Map<String, dynamic> data = formatJson(call.arguments);
          Map<String, dynamic> params = data['data'] ?? Map<String, dynamic>.from({});
          String type = data['type'];
          initSDKListenerList.forEach((key, value) {
            switch (type) {
              case 'onSelfInfoUpdated':
                final userInfo = V2TimUserFullInfo.fromJson(params);
                _catchListenerError(() {
                  value.onSelfInfoUpdated(userInfo);
                });
                break;
              case 'onConnectFailed':
                _catchListenerError(() {
                  value.onConnectFailed(
                    params['code'],
                    params['desc'],
                  );
                });
                break;
              case 'onConnecting':
                _catchListenerError(() {
                  value.onConnecting();
                });
                break;
              case 'onConnectSuccess':
                _catchListenerError(() {
                  value.onConnectSuccess();
                });
                break;
              case 'onKickedOffline':
                _catchListenerError(() {
                  value.onKickedOffline();
                });
                break;
              case 'onUserSigExpired':
                _catchListenerError(() {
                  value.onUserSigExpired();
                });
                break;
              case 'onUserStatusChanged':
                List<dynamic> statusList = params['statusList'];
                List<V2TimUserStatus> list = List.empty(growable: true);
                for (var element in statusList) {
                  list.add(V2TimUserStatus.fromJson(Map<String, dynamic>.from(element)));
                }
                _catchListenerError(() {
                  value.onUserStatusChanged(list);
                });
                break;
              case 'onLog':
                int level = params['level'];
                String log = params['content'];
                _catchListenerError(() {
                  value.onLog(level, log);
                });
                break;
              case "onExperimentalNotify":
                String key = params["key"];
                dynamic param = params["param"];
                _catchListenerError(() {
                  value.onExperimentalNotify(key, param);
                });
                break;
              case "onUserInfoChanged":
                List<dynamic> userInfoList = params['userInfoList'];
                List<V2TimUserFullInfo> list = List.empty(growable: true);
                for (var element in userInfoList) {
                  list.add(V2TimUserFullInfo.fromJson(Map<String, dynamic>.from(element)));
                }
                _catchListenerError(() {
                  value.onUserInfoChanged(list);
                });
                break;
              case "onAllReceiveMessageOptChanged":
                final receiveMessageOptInfo = V2TimReceiveMessageOptInfo.fromJson(params);
                _catchListenerError(() {
                  value.onAllReceiveMessageOptChanged(receiveMessageOptInfo);
                });
                break;
            }
          });
        } else if (call.method == ListenerType.groupListener) {
          Map<String, dynamic> data = formatJson(call.arguments);
          String type = data['type'];
          Map<String, dynamic> params = data['data'] == null ? Map<String, dynamic>.from({}) : Map<String, dynamic>.from(data['data']);

          String groupID = params['groupID'] ?? '';
          String opReason = params['opReason'] ?? '';
          bool isAgreeJoin = params['isAgreeJoin'] ?? false;
          String customData = params['customData'] ?? '';
          String topicID = params["topicID"] ?? "";
          List<String> topicIDList = params["topicIDList"] == null ? List.empty(growable: true) : List.castFrom(params["topicIDList"]);
          V2TimTopicInfo topicInfo = params["topicInfo"] == null ? V2TimTopicInfo() : V2TimTopicInfo.fromJson(params['topicInfo']);
          Map<String, String> groupAttributeMap = params['groupAttributeMap'] == null ? Map<String, String>.from({}) : Map<String, String>.from(params['groupAttributeMap']);

          List<Map<String, dynamic>> memberListMap = params['memberList'] == null ? List.empty(growable: true) : List.from(params['memberList']);

          List<Map<String, dynamic>> groupMemberChangeInfoListMap = params['groupMemberChangeInfoList'] == null ? List.empty(growable: true) : List.from(params['groupMemberChangeInfoList']);

          List<Map<String, dynamic>> groupChangeInfoListMap = params['groupChangeInfoList'] == null ? List.empty(growable: true) : List.from(params['groupChangeInfoList']);
          List<V2TimGroupChangeInfo> groupChangeInfoList = List.empty(growable: true);
          List<V2TimGroupMemberChangeInfo> groupMemberChangeInfoList = List.empty(growable: true);
          List<V2TimGroupMemberInfo> memberList = List.empty(growable: true);

          if (memberListMap.isNotEmpty) {
            for (var element in memberListMap) {
              memberList.add(V2TimGroupMemberInfo.fromJson(element));
            }
          }
          if (groupMemberChangeInfoListMap.isNotEmpty) {
            for (var element in groupMemberChangeInfoListMap) {
              groupMemberChangeInfoList.add(V2TimGroupMemberChangeInfo.fromJson(element));
            }
          }
          if (groupChangeInfoListMap.isNotEmpty) {
            for (var element in groupChangeInfoListMap) {
              groupChangeInfoList.add(V2TimGroupChangeInfo.fromJson(element));
            }
          }
          late V2TimGroupMemberInfo opUser;
          late V2TimGroupMemberInfo member;
          if (params['opUser'] != null) {
            opUser = V2TimGroupMemberInfo.fromJson(params['opUser']);
          }
          if (params['member'] != null) {
            member = V2TimGroupMemberInfo.fromJson(params['member']);
          }
          groupListenerList.forEach((key, value) {
            switch (type) {
              case 'onMemberEnter':
                _catchListenerError(() {
                  value.onMemberEnter(
                    groupID,
                    memberList,
                  );
                });
                break;
              case 'onMemberLeave':
                _catchListenerError(() {
                  value.onMemberLeave(
                    groupID,
                    member,
                  );
                });
                break;
              case 'onMemberInvited':
                _catchListenerError(() {
                  value.onMemberInvited(
                    groupID,
                    opUser,
                    memberList,
                  );
                });
                break;
              case 'onMemberKicked':
                _catchListenerError(() {
                  value.onMemberKicked(
                    groupID,
                    opUser,
                    memberList,
                  );
                });
                break;
              case 'onMemberInfoChanged':
                _catchListenerError(() {
                  value.onMemberInfoChanged(
                    groupID,
                    groupMemberChangeInfoList,
                  );
                });
                break;
              case 'onGroupCreated':
                _catchListenerError(() {
                  value.onGroupCreated(groupID);
                });
                break;
              case 'onGroupDismissed':
                _catchListenerError(() {
                  value.onGroupDismissed(
                    groupID,
                    opUser,
                  );
                });
                break;
              case 'onGroupRecycled':
                _catchListenerError(() {
                  value.onGroupRecycled(
                    groupID,
                    opUser,
                  );
                });
                break;
              case 'onGroupInfoChanged':
                _catchListenerError(() {
                  value.onGroupInfoChanged(
                    groupID,
                    groupChangeInfoList,
                  );
                });
                break;
              case 'onReceiveJoinApplication':
                _catchListenerError(() {
                  value.onReceiveJoinApplication(
                    groupID,
                    member,
                    opReason,
                  );
                });
                break;
              case 'onApplicationProcessed':
                _catchListenerError(() {
                  value.onApplicationProcessed(
                    groupID,
                    opUser,
                    isAgreeJoin,
                    opReason,
                  );
                });
                break;
              case 'onGrantAdministrator':
                _catchListenerError(() {
                  value.onGrantAdministrator(
                    groupID,
                    opUser,
                    memberList,
                  );
                });
                break;
              case 'onRevokeAdministrator':
                _catchListenerError(() {
                  value.onRevokeAdministrator(
                    groupID,
                    opUser,
                    memberList,
                  );
                });
                break;
              case 'onQuitFromGroup':
                _catchListenerError(() {
                  value.onQuitFromGroup(groupID);
                });
                break;
              case 'onReceiveRESTCustomData':
                _catchListenerError(() {
                  value.onReceiveRESTCustomData(
                    groupID,
                    customData,
                  );
                });
                break;
              case 'onGroupAttributeChanged':
                _catchListenerError(() {
                  value.onGroupAttributeChanged(
                    groupID,
                    groupAttributeMap,
                  );
                });
                break;
              case "onTopicCreated":
                _catchListenerError(() {
                  value.onTopicCreated(
                    groupID,
                    topicID,
                  );
                });
                break;
              case "onTopicDeleted":
                _catchListenerError(() {
                  value.onTopicDeleted(
                    groupID,
                    topicIDList,
                  );
                });
                break;
              case "onTopicInfoChanged":
                _catchListenerError(() {
                  value.onTopicInfoChanged(
                    groupID,
                    topicInfo,
                  );
                });
                break;
              case "onGroupCounterChanged":
                String key = params["key"] ?? "";
                int vv = params["value"] ?? 0;
                _catchListenerError(() {
                  value.onGroupCounterChanged(
                    groupID,
                    key,
                    vv,
                  );
                });
                break;
              case "onAllGroupMembersMuted":
                bool isMute = params["isMute"] ?? false;
                _catchListenerError(() {
                  value.onAllGroupMembersMuted(
                    groupID,
                    isMute,
                  );
                });
                break;
              case "onMemberMarkChanged":
                bool enableMark = params["enableMark"] ?? false;
                List<String> memberIDList = List.from(params["memberIDList"] ?? []);
                int markType = params["markType"] ?? 0;

                _catchListenerError(() {
                  value.onMemberMarkChanged(
                    groupID,
                    memberIDList,
                    markType,
                    enableMark,
                  );
                });
                break;
            }
          });
        } else if (call.method == ListenerType.advancedMsgListener) {
          Map<String, dynamic> data = formatJson(call.arguments);
          // String listenerUuid = data['listenerUuid'];
          // V2TimAdvancedMsgListener? listener =
          //     this.advancedMsgListenerList[listenerUuid];

          String type = data['type'];
          dynamic params = data['data'] ?? Map<String, dynamic>.from({});

          switch (type) {
            case 'onRecvNewMessage':
              advancedMsgListenerList.forEach((key, listener) {
                _catchListenerError(() {
                  listener.onRecvNewMessage(V2TimMessage.fromJson(params));
                });
              });

              break;
            case 'onRecvMessageModified':
              advancedMsgListenerList.forEach((key, listener) {
                _catchListenerError(() {
                  listener.onRecvMessageModified(V2TimMessage.fromJson(params));
                });
              });
              break;
            case 'onRecvC2CReadReceipt':
              List dataList = params;
              List<V2TimMessageReceipt> receiptList = List.empty(growable: true);
              for (var element in dataList) {
                receiptList.add(V2TimMessageReceipt.fromJson(element));
              }

              advancedMsgListenerList.forEach((key, listener) {
                _catchListenerError(() {
                  listener.onRecvC2CReadReceipt(receiptList);
                });
              });
              break;
            case 'onRecvMessageRevoked':
              advancedMsgListenerList.forEach((key, listener) {
                _catchListenerError(() {
                  listener.onRecvMessageRevoked(params);
                });
              });
              break;
            case 'onSendMessageProgress':
              final message = V2TimMessage.fromJson(params['message']);

              advancedMsgListenerList.forEach((key, listener) {
                _catchListenerError(() {
                  listener.onSendMessageProgress(
                    message,
                    params['progress'],
                  );
                });
              });
              break;
            case 'onRecvMessageReadReceipts':
              List dataList = params;
              List<V2TimMessageReceipt> receiptList = List.empty(growable: true);
              for (var element in dataList) {
                receiptList.add(V2TimMessageReceipt.fromJson(element));
              }

              advancedMsgListenerList.forEach((key, listener) {
                _catchListenerError(() {
                  listener.onRecvMessageReadReceipts(receiptList);
                });
              });
              break;
            case "onRecvMessageExtensionsChanged":
              Map<String, dynamic> cbdata = Map<String, Object>.from(params);
              String msgID = cbdata["msgID"] ?? "";
              List<Map<String, dynamic>> extensions = List.from(cbdata["extensions"] ?? []);
              List<V2TimMessageExtension> resList = List.empty(growable: true);
              for (var element in extensions) {
                resList.add(V2TimMessageExtension(
                  extensionKey: element["extensionKey"] ?? "",
                  extensionValue: element["extensionValue"] ?? "",
                ));
              }

              advancedMsgListenerList.forEach((key, listener) {
                _catchListenerError(() {
                  listener.onRecvMessageExtensionsChanged(
                    msgID,
                    resList,
                  );
                });
              });
              break;
            case "onRecvMessageExtensionsDeleted":
              Map<String, dynamic> cbdata = Map<String, dynamic>.from(params);
              String msgID = cbdata["msgID"] ?? "";
              List<String> extensionKeys = List.from(cbdata["extensionKeys"] ?? []);

              advancedMsgListenerList.forEach((key, listener) {
                _catchListenerError(() {
                  listener.onRecvMessageExtensionsDeleted(
                    msgID,
                    extensionKeys,
                  );
                });
              });
              break;
            case "onMessageDownloadProgressCallback":
              Map<String, dynamic> cbdata = Map<String, dynamic>.from(params);

              advancedMsgListenerList.forEach((key, listener) {
                _catchListenerError(() {
                  listener.onMessageDownloadProgressCallback(
                    V2TimMessageDownloadProgress.fromJson(cbdata),
                  );
                });
              });
              break;
            case "onRecvMessageReactionsChanged":
              List<Map<String, dynamic>> changeInfosm = List<Map<String, dynamic>>.from(params["changeInfos"]);
              List<V2TIMMessageReactionChangeInfo> infos = List.empty(growable: true);
              changeInfosm.forEach(((element) {
                infos.add(V2TIMMessageReactionChangeInfo.fromJson(element));
              }));

              advancedMsgListenerList.forEach((key, listener) {
                _catchListenerError(() {
                  listener.onRecvMessageReactionsChanged(infos);
                });
              });
              break;
            case "onRecvMessageRevokedWithInfo":
              String msgID = params["msgID"];
              V2TimUserFullInfo operateUser = V2TimUserFullInfo.fromJson(params['operateUser'] ?? {});
              String reason = params["reason"];

              advancedMsgListenerList.forEach((key, listener) {
                _catchListenerError(() {
                  listener.onRecvMessageRevokedWithInfo(msgID, operateUser, reason);
                });
              });
              break;
          }
        } else if (call.method == ListenerType.conversationListener) {
          Map<String, dynamic> data = formatJson(call.arguments);
          String type = data['type'];

          conversationListenerList.forEach((key, value) {
            switch (type) {
              case 'onSyncServerStart':
                _catchListenerError(() {
                  value.onSyncServerStart();
                });
                break;
              case 'onSyncServerFinish':
                _catchListenerError(() {
                  value.onSyncServerFinish();
                });
                break;
              case 'onSyncServerFailed':
                _catchListenerError(() {
                  value.onSyncServerFailed();
                });
                break;
              case 'onNewConversation':
                dynamic params = data['data'] == null ? List.empty(growable: true) : List.from(data['data']);
                List<V2TimConversation> conversationList = List.empty(growable: true);
                params.forEach((element) {
                  conversationList.add(V2TimConversation.fromJson(element));
                });
                _catchListenerError(() {
                  value.onNewConversation(conversationList);
                });
                break;
              case 'onConversationChanged':
                dynamic params = data['data'] == null ? List.empty(growable: true) : List.from(data['data']);
                List<V2TimConversation> conversationList = List.empty(growable: true);
                params.forEach((element) {
                  conversationList.add(V2TimConversation.fromJson(element));
                });

                _catchListenerError(() {
                  value.onConversationChanged(conversationList);
                });
                break;
              case 'onTotalUnreadMessageCountChanged':
                dynamic params = data['data'] ?? 0;
                _catchListenerError(() {
                  value.onTotalUnreadMessageCountChanged(params);
                });
                break;
              case "onConversationGroupCreated":
                Map<String, dynamic> param = data["data"];
                String groupName = param["groupName"];
                List<V2TimConversation> list = List.empty(growable: true);
                for (var v in List.from(param["conversationList"])) {
                  list.add(V2TimConversation.fromJson(v));
                }
                _catchListenerError(() {
                  value.onConversationGroupCreated(groupName, list);
                });
                break;
              case "onConversationGroupDeleted":
                _catchListenerError(() {
                  value.onConversationGroupDeleted(data["data"]);
                });
                break;
              case "onConversationGroupNameChanged":
                Map<String, dynamic> param = data["data"];
                _catchListenerError(() {
                  value.onConversationGroupNameChanged(param["oldName"], param["newName"]);
                });
                break;
              case "onConversationsAddedToGroup":
                Map<String, dynamic> param = data["data"];
                String groupName = param["groupName"];
                List<V2TimConversation> list = List.empty(growable: true);
                for (var v in List.from(param["conversationList"])) {
                  list.add(V2TimConversation.fromJson(v));
                }
                _catchListenerError(() {
                  value.onConversationsAddedToGroup(groupName, list);
                });
                break;
              case "onConversationsDeletedFromGroup":
                Map<String, dynamic> param = data["data"];
                String groupName = param["groupName"];
                List<V2TimConversation> list = List.empty(growable: true);
                for (var v in List.from(param["conversationList"])) {
                  list.add(V2TimConversation.fromJson(v));
                }
                _catchListenerError(() {
                  value.onConversationsDeletedFromGroup(groupName, list);
                });
                break;
              case "onConversationDeleted":
                List<String> list = List<String>.from(data["data"] ?? []);
                _catchListenerError(() {
                  value.onConversationDeleted(list);
                });
                break;
              case "onUnreadMessageCountChangedByFilter":
                Map<String, dynamic> param = data["data"];
                V2TimConversationFilter filter = V2TimConversationFilter.fromJson(param["filter"] ?? {});
                int totalUnreadCount = param["totalUnreadCount"] ?? 0;
                _catchListenerError(() {
                  value.onUnreadMessageCountChangedByFilter(filter, totalUnreadCount);
                });
                break;
            }
          });
        } else if (call.method == ListenerType.friendListener) {
          Map<String, dynamic> data = formatJson(call.arguments);
          String type = data['type'];
          dynamic params = data['data'] ?? Map<String, dynamic>.from({});
          friendListenerList.forEach((key, value) {
            switch (type) {
              case 'onFriendApplicationListAdded':
                List applicationListMap = params;
                List<V2TimFriendApplication> applicationList = List.empty(growable: true);
                for (var element in applicationListMap) {
                  applicationList.add(V2TimFriendApplication.fromJson(element));
                }
                _catchListenerError(() {
                  value.onFriendApplicationListAdded(applicationList);
                });
                break;
              case 'onFriendApplicationListDeleted':
                List<String> userIDList = List.from(params);
                _catchListenerError(() {
                  value.onFriendApplicationListDeleted(userIDList);
                });
                break;
              case 'onFriendApplicationListRead':
                _catchListenerError(() {
                  value.onFriendApplicationListRead();
                });
                break;
              case 'onFriendListAdded':
                List userMap = params;
                List<V2TimFriendInfo> users = List.empty(growable: true);
                for (var element in userMap) {
                  users.add(V2TimFriendInfo.fromJson(element));
                }
                _catchListenerError(() {
                  value.onFriendListAdded(users);
                });
                break;
              case 'onFriendListDeleted':
                List<String> userList = List.from(params);
                _catchListenerError(() {
                  value.onFriendListDeleted(userList);
                });
                break;
              case 'onBlackListAdd':
                List infoListMap = params;
                List<V2TimFriendInfo> infoList = List.empty(growable: true);
                for (var element in infoListMap) {
                  infoList.add(V2TimFriendInfo.fromJson(element));
                }
                _catchListenerError(() {
                  value.onBlackListAdd(infoList);
                });
                break;
              case 'onBlackListDeleted':
                List<String> userList = List.from(params);
                _catchListenerError(() {
                  value.onBlackListDeleted(userList);
                });
                break;
              case 'onFriendInfoChanged':
                List infoListMap = params;
                List<V2TimFriendInfo> infoList = List.empty(growable: true);
                for (var element in infoListMap) {
                  infoList.add(V2TimFriendInfo.fromJson(element));
                }
                _catchListenerError(() {
                  value.onFriendInfoChanged(infoList);
                });
                break;
            }
          });
        } else if (call.method == 'logFromSwift') {
        } else if (call.method == ListenerType.signalingListener) {
          Map<String, dynamic> d = formatJson(call.arguments);
          String type = d['type'];
          Map<String, dynamic> params = d['data'];
          String inviteID = params['inviteID'] ?? '';
          String inviter = params['inviter'] ?? '';
          String groupID = params['groupID'] ?? '';
          List<String>? inviteeList = params['inviteeList'] == null ? null : List.from(params['inviteeList']);
          String data = params['data'] ?? '';
          String invitee = params['invitee'] ?? '';
          signalingListenerList.forEach((key, value) {
            switch (type) {
              case 'onReceiveNewInvitation':
                _catchListenerError(() {
                  value.onReceiveNewInvitation(inviteID, inviter, groupID, inviteeList!, data);
                });
                break;
              case 'onInviteeAccepted':
                _catchListenerError(() {
                  value.onInviteeAccepted(inviteID, invitee, data);
                });
                break;
              case 'onInviteeRejected':
                _catchListenerError(() {
                  value.onInviteeRejected(inviteID, invitee, data);
                });
                break;
              case 'onInvitationCancelled':
                _catchListenerError(() {
                  value.onInvitationCancelled(inviteID, inviter, data);
                });
                break;
              case 'onInvitationTimeout':
                _catchListenerError(() {
                  value.onInvitationTimeout(inviteID, inviteeList!);
                });
                break;
            }
          });
        }
      } catch (err) {
        debugPrint("重点关注，回调失败了，数据类型异常。$err ${call.method} ${call.arguments['type']} ${call.arguments['data']}");
      }
    });
  }

  @override
  Future<void> emitUIKitEvent(UIKitEvent event) async {
    initSDKListenerList.forEach((key, value) {
      value.onUIKitEventEmited(event);
    });
  }

  @override
  Future<void> emitPluginEvent(PluginEvent event) async {
    initSDKListenerList.forEach((key, value) {
      value.onPluginEventEmited(event);
    });
  }

  @override
  Future<V2TimValueCallback<bool>> initSDK({
    required int sdkAppID,
    required int loglevel,
    required V2TimSDKListener listener,
    required int uiPlatform,
    bool? showImLog,
    List<V2TimPlugins>? plugins,
  }) async {
    final String uuid = Utils.generateUniqueString();
    initSDKListenerList[uuid] = listener;
    final Directory tempDir = await getTemporaryDirectory();
    String tp = tempDir.path;
    var imDirec = Directory("$tp\\TencentCloudChat\\");
    var imConfigDirec = Directory("$tp\\TencentCloudChat\\Config\\");
    var imLogDirec = Directory("$tp\\TencentCloudChat/Log\\");
    var imDownLoadDirec = Directory("$tp\\TencentCloudChat\\DownLoad\\");
    if (!imDirec.existsSync()) {
      imDirec.createSync(recursive: true);
    }
    if (!imConfigDirec.existsSync()) {
      imConfigDirec.createSync(recursive: true);
    }
    if (!imLogDirec.existsSync()) {
      imLogDirec.createSync(recursive: true);
    }
    if (!imDownLoadDirec.existsSync()) {
      imDownLoadDirec.createSync(recursive: true);
    }
    return V2TimValueCallback<bool>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'initSDK',
          buildTimManagerParam(
            {
              "sdkAppID": sdkAppID,
              "logLevel": loglevel,
              "listenerUuid": uuid,
              "uiPlatform": uiPlatform,
              "showImLog": showImLog,
              "plugins": plugins?.map((e) => e.index.toString()).toList() ?? [],
              "tempPath": imDirec.path,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> unInitSDK() async {
    initSDKListenerList.clear();
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'unInitSDK',
          buildTimManagerParam(
            {},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<String>> getVersion() async {
    return V2TimValueCallback<String>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'getVersion',
          buildTimManagerParam(
            {},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<int>> getServerTime() async {
    return V2TimValueCallback<int>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'getServerTime',
          buildTimManagerParam(
            {},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> login({
    required String userID,
    required String userSig,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'login',
          buildTimManagerParam({
            "userID": userID,
            "userSig": userSig,
          }),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> logout() async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'logout',
          buildTimManagerParam(
            {},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessage>> sendC2CTextMessage({
    required String text,
    required String userID,
  }) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'sendC2CTextMessage',
          buildTimManagerParam(
            {
              "text": text,
              "userID": userID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessage>> sendC2CCustomMessage({
    required String customData,
    required String userID,
  }) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'sendC2CCustomMessage',
          buildTimManagerParam(
            {
              "customData": customData,
              "userID": userID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessage>> sendGroupTextMessage({
    required String text,
    required String groupID,
    int priority = 0,
  }) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "sendGroupTextMessage",
          buildTimManagerParam(
            {
              "text": text,
              "groupID": groupID,
              "priority": priority,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessage>> sendGroupCustomMessage({
    required String customData,
    required String groupID,
    int priority = 0,
  }) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "sendGroupCustomMessage",
          buildTimManagerParam(
            {
              "customData": customData,
              "groupID": groupID,
              "priority": priority,
            },
          ),
        ),
      ),
    );
  }

  /// 获取登录用户
  ///
  @override
  Future<V2TimValueCallback<String>> getLoginUser() async {
    return V2TimValueCallback<String>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'getLoginUser',
          buildTimManagerParam(
            {},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<int>> getLoginStatus() async {
    return V2TimValueCallback<int>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'getLoginStatus',
          buildTimManagerParam(
            {},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimUserFullInfo>>> getUsersInfo({
    required List<String> userIDList,
  }) async {
    return V2TimValueCallback<List<V2TimUserFullInfo>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getUsersInfo",
          buildTimManagerParam(
            {
              "userIDList": userIDList,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<String>> createGroup({
    String? groupID,
    required String groupType,
    required String groupName,
    String? notification,
    String? introduction,
    String? faceUrl,
    bool? isAllMuted,
    int? addOpt,
    List<V2TimGroupMember>? memberList,
    bool? isSupportTopic,
    int? approveOpt,
  }) async {
    return V2TimValueCallback<String>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "createGroup",
          buildGroupManagerParam(
            {
              "groupID": groupID,
              "groupType": groupType,
              "groupName": groupName,
              "notification": notification,
              "introduction": introduction,
              "faceUrl": faceUrl,
              "isAllMuted": isAllMuted,
              "addOpt": addOpt,
              "memberList": memberList?.map((e) => e.toJson()).toList(),
              "isSupportTopic": isSupportTopic,
              "approveOpt": approveOpt,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> joinGroup({
    required String groupID,
    required String message,
    String? groupType,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "joinGroup",
          buildTimManagerParam(
            {
              "groupID": groupID,
              "message": message,
              "groupType": groupType,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> quitGroup({
    required String groupID,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "quitGroup",
          buildTimManagerParam(
            {
              "groupID": groupID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> dismissGroup({
    required String groupID,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "dismissGroup",
          buildTimManagerParam(
            {
              "groupID": groupID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> setSelfInfo({
    required V2TimUserFullInfo userFullInfo,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "setSelfInfo",
          buildTimManagerParam(
            {
              "nickName": userFullInfo.nickName,
              "faceUrl": userFullInfo.faceUrl,
              "selfSignature": userFullInfo.selfSignature,
              "birthday": userFullInfo.birthday,
              "gender": userFullInfo.gender,
              "allowType": userFullInfo.allowType,
              "customInfo": userFullInfo.customInfo,
              "level": userFullInfo.level,
              "role": userFullInfo.role,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<Object>> callExperimentalAPI({
    required String api,
    Object? param,
  }) async {
    return V2TimValueCallback<Object>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "callExperimentalAPI",
          buildTimManagerParam(
            {
              "api": api,
              "param": param,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<void> addSimpleMsgListener({required V2TimSimpleMsgListener listener}) async {
    final uuid = Utils.generateUniqueString();
    simpleMessageListenerList[uuid] = listener;
    return _channel.invokeMethod("addSimpleMsgListener", buildTimManagerParam({"listenerUuid": uuid}));
  }

  @override
  Future<void> removeSimpleMsgListener({
    V2TimSimpleMsgListener? listener,
  }) async {
    var listenerUuid = "";
    if (listener != null) {
      listenerUuid = simpleMessageListenerList.keys.firstWhere((k) => simpleMessageListenerList[k] == listener, orElse: () => "");
      simpleMessageListenerList.remove(listenerUuid);
    } else {
      simpleMessageListenerList.clear();
    }
    return _channel.invokeMethod("removeSimpleMsgListener", buildTimManagerParam({"listenerUuid": listenerUuid}));
  }

  @override
  Future setAPNSListener() {
    return _channel.invokeMethod("setAPNSListener", buildTimManagerParam({}));
  }

  @override
  Future<V2TimValueCallback<V2TimConversationResult>> getConversationList({
    required String nextSeq,
    required int count,
  }) async {
    return V2TimValueCallback<V2TimConversationResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getConversationList",
          buildConversationManagerParam(
            {
              "nextSeq": nextSeq,
              "count": count,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<LinkedHashMap<dynamic, dynamic>> getConversationListWithoutFormat({
    required String nextSeq,
    required int count,
  }) async {
    return await _channel.invokeMethod(
      "getConversationList",
      buildConversationManagerParam(
        {
          "nextSeq": nextSeq,
          "count": count,
        },
      ),
    );
  }

  @override
  Future<void> removeConversationListener({
    V2TimConversationListener? listener,
  }) async {
    var listenerUuid = "";
    if (listener != null) {
      listenerUuid = conversationListenerList.keys.firstWhere(
        (k) => conversationListenerList[k] == listener,
        orElse: () => "",
      );
      conversationListenerList.remove(listenerUuid);
    } else {
      conversationListenerList.clear();
    }
    return _channel.invokeMethod("removeConversationListener", buildConversationManagerParam({"listenerUuid": listenerUuid}));
  }

  @override
  Future<void> removeFriendListener({
    V2TimFriendshipListener? listener,
  }) async {
    var listenerUuid = "";
    if (listener != null) {
      listenerUuid = friendListenerList.keys.firstWhere(
        (k) => friendListenerList[k] == listener,
        orElse: () => "",
      );
      friendListenerList.remove(listenerUuid);
    } else {
      friendListenerList.clear();
    }
    return _channel.invokeMethod("removeFriendListener", buildFriendManagerParam({"listenerUuid": listenerUuid}));
  }

  @override
  Future<void> removeGroupListener({
    V2TimGroupListener? listener,
  }) async {
    var listenerUuid = "";
    if (listener != null) {
      listenerUuid = groupListenerList.keys.firstWhere(
        (k) => groupListenerList[k] == listener,
        orElse: () => "",
      );
      groupListenerList.remove(listenerUuid);
    } else {
      groupListenerList.clear();
    }
    return _channel.invokeMethod("removeGroupListener", buildTimManagerParam({"listenerUuid": listenerUuid}));
  }

  @override
  Future<void> addConversationListener({
    required V2TimConversationListener listener,
  }) async {
    final String uuid = Utils.generateUniqueString();
    conversationListenerList[uuid] = listener;
    return await _channel.invokeMethod(
      "addConversationListener",
      buildConversationManagerParam(
        {
          "listenerUuid": uuid,
        },
      ),
    );
  }

  @override
  Future<void> addGroupListener({
    required V2TimGroupListener listener,
  }) async {
    final uuid = Utils.generateUniqueString();
    groupListenerList[uuid] = listener;
    await _channel.invokeMethod(
      "addGroupListener",
      buildTimManagerParam(
        {
          "listenerUuid": uuid,
        },
      ),
    );
  }

  @override
  Future<void> addFriendListener({
    required V2TimFriendshipListener listener,
  }) async {
    final String uuid = Utils.generateUniqueString();
    friendListenerList[uuid] = listener;
    return await _channel.invokeMethod(
      "addFriendListener",
      buildFriendManagerParam(
        {
          "listenerUuid": uuid,
        },
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimConversation>>> getConversationListByConversaionIds({
    required List<String> conversationIDList,
  }) async {
    return V2TimValueCallback<List<V2TimConversation>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getConversationListByConversaionIds",
          buildConversationManagerParam(
            {
              "conversationIDList": conversationIDList,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<void> setConversationListener({
    required V2TimConversationListener listener,
  }) async {
    final String uuid = Utils.generateUniqueString();
    conversationListenerList[uuid] = listener;
    return _channel.invokeMethod("setConversationListener", buildConversationManagerParam({"listenerUuid": uuid}));
  }

  @override
  Future<V2TimValueCallback<V2TimConversation>> getConversation({
    /*required*/ required String conversationID,
  }) async {
    return V2TimValueCallback<V2TimConversation>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'getConversation',
          buildConversationManagerParam(
            {
              "conversationID": conversationID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> deleteConversation({
    /*required*/ required String conversationID,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "deleteConversation",
          buildConversationManagerParam(
            {
              "conversationID": conversationID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> setConversationDraft({
    required String conversationID,
    String? draftText,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "setConversationDraft",
          buildConversationManagerParam(
            {
              "conversationID": conversationID,
              "draftText": draftText,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> pinConversation({
    required String conversationID,
    required bool isPinned,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "pinConversation",
          buildConversationManagerParam(
            {
              "conversationID": conversationID,
              "isPinned": isPinned,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<int>> getTotalUnreadMessageCount() async {
    return V2TimValueCallback<int>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getTotalUnreadMessageCount",
          buildConversationManagerParam(
            {},
          ),
        ),
      ),
    );
  }

  /* *****-好友关系模块-******** */
  /*
  *      ┌─┐       ┌─┐
  *   ┌──┘ ┴───────┘ ┴──┐
  *   │                 │
  *   │       ───       │
  *   │   >        <    │
  *   │                 │
  *   │   ...  ⌒  ...   │
  *   │                 │
  *   └───┐         ┌───┘
  *       │         │
  *       │         │
  *       │         │
  *       │         └──────────────┐
  *       │                        │
  *       │                        ├─┐
  *       │                        ┌─┘
  *       │                        │
  *       └─┐  ┐  ┌───────┬──┐  ┌──┘
  *         │ ─┤ ─┤       │ ─┤ ─┤
  *         └──┴──┘       └──┴──┘
  *                神兽保佑，不出bug
  *               只是为了区分不同模块
   */
  @override
  Future<V2TimValueCallback<List<V2TimFriendInfo>>> getFriendList() async {
    return V2TimValueCallback<List<V2TimFriendInfo>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getFriendList",
          buildFriendManagerParam(
            {},
          ),
        ),
      ),
    );
  }

  @override
  Future<void> setFriendListener({
    required V2TimFriendshipListener listener,
  }) async {
    final String listenerUuid = Utils.generateUniqueString();
    friendListenerList[listenerUuid] = listener;
    return _channel.invokeMethod("setFriendListener", buildFriendManagerParam({"listenerUuid": listenerUuid}));
  }

  @override
  Future<V2TimValueCallback<List<V2TimFriendInfoResult>>> getFriendsInfo({
    required List<String> userIDList,
  }) async {
    return V2TimValueCallback<List<V2TimFriendInfoResult>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getFriendsInfo",
          buildFriendManagerParam(
            {
              "userIDList": userIDList,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimFriendOperationResult>> addFriend({
    required String userID,
    String? remark,
    String? friendGroup,
    String? addWording,
    String? addSource,
    required int addType,
  }) async {
    return V2TimValueCallback<V2TimFriendOperationResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "addFriend",
          buildFriendManagerParam(
            {
              "userID": userID,
              "remark": remark,
              "friendGroup": friendGroup,
              "addWording": addWording,
              "addSource": addSource,
              "addType": addType,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> setFriendInfo({
    required String userID,
    String? friendRemark,
    Map<String, String>? friendCustomInfo,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "setFriendInfo",
          buildFriendManagerParam(
            {
              "userID": userID,
              "friendRemark": friendRemark,
              "friendCustomInfo": friendCustomInfo,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimFriendOperationResult>>> deleteFromFriendList({
    required List<String> userIDList,
    required int deleteType,
  }) async {
    return V2TimValueCallback<List<V2TimFriendOperationResult>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "deleteFromFriendList",
          buildFriendManagerParam(
            {
              "userIDList": userIDList,
              "deleteType": deleteType,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimFriendCheckResult>>> checkFriend({
    required List<String> userIDList,
    required int checkType,
  }) async {
    return V2TimValueCallback<List<V2TimFriendCheckResult>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "checkFriend",
          buildFriendManagerParam(
            {
              "userIDList": userIDList,
              "checkType": checkType,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimFriendApplicationResult>> getFriendApplicationList() async {
    return V2TimValueCallback<V2TimFriendApplicationResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getFriendApplicationList",
          buildFriendManagerParam(
            {},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimFriendOperationResult>> acceptFriendApplication({
    required int responseType,
    required int type,
    required String userID,
  }) async {
    return V2TimValueCallback<V2TimFriendOperationResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "acceptFriendApplication",
          buildFriendManagerParam(
            {
              "responseType": responseType,
              "type": type,
              "userID": userID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimFriendOperationResult>> refuseFriendApplication({
    required int type,
    required String userID,
  }) async {
    return V2TimValueCallback<V2TimFriendOperationResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "refuseFriendApplication",
          buildFriendManagerParam(
            {
              "type": type,
              "userID": userID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> deleteFriendApplication({
    required int type,
    required String userID,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "deleteFriendApplication",
          buildFriendManagerParam(
            {
              "type": type,
              "userID": userID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> setFriendApplicationRead() async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "setFriendApplicationRead",
          buildFriendManagerParam(
            {},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimFriendInfo>>> getBlackList() async {
    return V2TimValueCallback<List<V2TimFriendInfo>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getBlackList",
          buildFriendManagerParam(
            {},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimFriendOperationResult>>> addToBlackList({
    required List<String> userIDList,
  }) async {
    return V2TimValueCallback<List<V2TimFriendOperationResult>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "addToBlackList",
          buildFriendManagerParam(
            {
              "userIDList": userIDList,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimFriendOperationResult>>> deleteFromBlackList({
    required List<String> userIDList,
  }) async {
    return V2TimValueCallback<List<V2TimFriendOperationResult>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "deleteFromBlackList",
          buildFriendManagerParam(
            {
              "userIDList": userIDList,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimFriendOperationResult>>> createFriendGroup({
    required String groupName,
    List<String>? userIDList,
  }) async {
    return V2TimValueCallback<List<V2TimFriendOperationResult>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "createFriendGroup",
          buildFriendManagerParam(
            {
              "groupName": groupName,
              "userIDList": userIDList,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimFriendGroup>>> getFriendGroups({
    List<String>? groupNameList,
  }) async {
    return V2TimValueCallback<List<V2TimFriendGroup>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getFriendGroups",
          buildFriendManagerParam(
            {
              "groupNameList": groupNameList,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> deleteFriendGroup({
    required List<String> groupNameList,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "deleteFriendGroup",
          buildFriendManagerParam(
            {
              "groupNameList": groupNameList,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> renameFriendGroup({
    required String oldName,
    required String newName,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "renameFriendGroup",
          buildFriendManagerParam(
            {
              "oldName": oldName,
              "newName": newName,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimFriendOperationResult>>> addFriendsToFriendGroup({
    required String groupName,
    required List<String> userIDList,
  }) async {
    return V2TimValueCallback<List<V2TimFriendOperationResult>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "addFriendsToFriendGroup",
          buildFriendManagerParam(
            {
              "groupName": groupName,
              "userIDList": userIDList,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimFriendOperationResult>>> deleteFriendsFromFriendGroup({
    required String groupName,
    required List<String> userIDList,
  }) async {
    return V2TimValueCallback<List<V2TimFriendOperationResult>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "deleteFriendsFromFriendGroup",
          buildFriendManagerParam(
            {
              "groupName": groupName,
              "userIDList": userIDList,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimFriendInfoResult>>> searchFriends({
    required V2TimFriendSearchParam searchParam,
  }) async {
    return V2TimValueCallback<List<V2TimFriendInfoResult>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "searchFriends",
          buildFriendManagerParam(
            {
              "searchParam": searchParam.toJson(),
            },
          ),
        ),
      ),
    );
  }
  /* *****-群组模块-******** */
  /*
  *      ┌─┐       ┌─┐
  *   ┌──┘ ┴───────┘ ┴──┐
  *   │                 │
  *   │       ───       │
  *   │   >        <    │
  *   │                 │
  *   │   ...  ⌒  ...   │
  *   │                 │
  *   └───┐         ┌───┘
  *       │         │
  *       │         │
  *       │         │
  *       │         └──────────────┐
  *       │                        │
  *       │                        ├─┐
  *       │                        ┌─┘
  *       │                        │
  *       └─┐  ┐  ┌───────┬──┐  ┌──┘
  *         │ ─┤ ─┤       │ ─┤ ─┤
  *         └──┴──┘       └──┴──┘
  *                神兽保佑，不出bug
  *               只是为了区分不同模块
   */

  @override
  Future<V2TimValueCallback<List<V2TimGroupInfo>>> getJoinedGroupList() async {
    return V2TimValueCallback<List<V2TimGroupInfo>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getJoinedGroupList",
          buildGroupManagerParam(
            {},
          ),
        ),
      ),
    );
  }

  @override
  Future<void> setGroupListener({
    required V2TimGroupListener listener,
  }) async {
    final uuid = Utils.generateUniqueString();
    groupListenerList[uuid] = listener;
    return _channel.invokeMethod("setGroupListener", buildTimManagerParam({"listenerUuid": uuid}));
  }

  @override
  Future<V2TimValueCallback<List<V2TimGroupInfoResult>>> getGroupsInfo({
    required List<String> groupIDList,
  }) async {
    return V2TimValueCallback<List<V2TimGroupInfoResult>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getGroupsInfo",
          buildGroupManagerParam(
            {
              "groupIDList": groupIDList,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> setGroupInfo({
    required V2TimGroupInfo info,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "setGroupInfo",
          buildGroupManagerParam(
            {
              "groupID": info.groupID,
              "groupType": info.groupType,
              "groupName": info.groupName,
              "notification": info.notification,
              "introduction": info.introduction,
              "faceUrl": info.faceUrl,
              "isAllMuted": info.isAllMuted,
              "addOpt": info.groupAddOpt,
              "customInfo": info.customInfo,
              "isSupportTopic": info.isSupportTopic,
              "approveOpt": info.approveOpt,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> setGroupAttributes({
    required String groupID,
    required Map<String, String> attributes,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "setGroupAttributes",
          buildGroupManagerParam(
            {
              "groupID": groupID,
              "attributes": attributes,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> deleteGroupAttributes({
    required String groupID,
    required List<String> keys,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "deleteGroupAttributes",
          buildGroupManagerParam(
            {
              "groupID": groupID,
              "keys": keys,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<Map<String, String>>> getGroupAttributes({
    required String groupID,
    List<String>? keys,
  }) async {
    return V2TimValueCallback<Map<String, String>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getGroupAttributes",
          buildGroupManagerParam(
            {
              "groupID": groupID,
              "keys": keys,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<int>> getGroupOnlineMemberCount({
    required String groupID,
  }) async {
    return V2TimValueCallback<int>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getGroupOnlineMemberCount",
          buildGroupManagerParam(
            {
              "groupID": groupID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimGroupMemberInfoResult>> getGroupMemberList({
    required String groupID,
    required int filter,
    required String nextSeq,
    int count = 15,
    int offset = 0,
  }) async {
    return V2TimValueCallback<V2TimGroupMemberInfoResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getGroupMemberList",
          buildGroupManagerParam(
            {"groupID": groupID, "filter": filter, "nextSeq": nextSeq, "count": count, "offset": offset},
          ),
        ),
      ),
    );
  }

  @override

  ///获取指定的群成员资料
  ///
  Future<V2TimValueCallback<List<V2TimGroupMemberFullInfo>>> getGroupMembersInfo({
    required String groupID,
    required List<String> memberList,
  }) async {
    return V2TimValueCallback<List<V2TimGroupMemberFullInfo>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getGroupMembersInfo",
          buildGroupManagerParam(
            {
              "groupID": groupID,
              "memberList": memberList,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> setGroupMemberInfo({
    required String groupID,
    required String userID,
    String? nameCard,
    Map<String, String>? customInfo,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "setGroupMemberInfo",
          buildGroupManagerParam(
            {
              "userID": userID,
              "groupID": groupID,
              "nameCard": nameCard ?? "",
              "customInfo": customInfo,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> muteGroupMember({
    required String groupID,
    required String userID,
    required int seconds,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "muteGroupMember",
          buildGroupManagerParam(
            {
              "groupID": groupID,
              "userID": userID,
              "seconds": seconds,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimGroupMemberOperationResult>>> inviteUserToGroup({
    required String groupID,
    required List<String> userList,
  }) async {
    return V2TimValueCallback<List<V2TimGroupMemberOperationResult>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "inviteUserToGroup",
          buildGroupManagerParam(
            {
              "groupID": groupID,
              "userList": userList,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> kickGroupMember({
    required String groupID,
    required List<String> memberList,
    int? duration,
    String? reason,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "kickGroupMember",
          buildGroupManagerParam(
            {
              "groupID": groupID,
              "memberList": memberList,
              "reason": reason,
              "duration": duration ?? 0,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> setGroupMemberRole({
    required String groupID,
    required String userID,
    required int role,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "setGroupMemberRole",
          buildGroupManagerParam(
            {
              "groupID": groupID,
              "userID": userID,
              "role": role,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> transferGroupOwner({
    required String groupID,
    required String userID,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "transferGroupOwner",
          buildGroupManagerParam(
            {
              "groupID": groupID,
              "userID": userID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimGroupApplicationResult>> getGroupApplicationList() async {
    return V2TimValueCallback<V2TimGroupApplicationResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getGroupApplicationList",
          buildGroupManagerParam(
            {},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> acceptGroupApplication({
    required String groupID,
    String? reason,
    required String fromUser,
    required String toUser,
    int? addTime,
    int? type,
    String? webMessageInstance,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "acceptGroupApplication",
          buildGroupManagerParam(
            {"groupID": groupID, "reason": reason, "fromUser": fromUser, "toUser": toUser, "addTime": addTime, "type": type, "webMessageInstance": webMessageInstance},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> refuseGroupApplication({
    required String groupID,
    String? reason,
    required String fromUser,
    required String toUser,
    required int addTime,
    required int type,
    String? webMessageInstance,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "refuseGroupApplication",
          buildGroupManagerParam(
            {"groupID": groupID, "reason": reason, "fromUser": fromUser, "toUser": toUser, "addTime": addTime, "type": type, "webMessageInstance": webMessageInstance},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> setGroupApplicationRead() async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "setGroupApplicationRead",
          buildGroupManagerParam(
            {},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimGroupInfo>>> searchGroups({
    required V2TimGroupSearchParam searchParam,
  }) async {
    return V2TimValueCallback<List<V2TimGroupInfo>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "searchGroups",
          buildGroupManagerParam(
            {
              "searchParam": searchParam.toJson(),
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2GroupMemberInfoSearchResult>> searchGroupMembers({
    required V2TimGroupMemberSearchParam param,
  }) async {
    return V2TimValueCallback<V2GroupMemberInfoSearchResult>.fromJson(
      formatJson(await _channel.invokeMethod(
        "searchGroupMembers",
        buildGroupManagerParam(
          {
            "param": param.toJson(),
          },
        ),
      )),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimGroupInfo>> searchGroupByID({
    required String groupID,
  }) async {
    return V2TimValueCallback<V2TimGroupInfo>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "searchGroupByID",
          buildGroupManagerParam(
            {"groupID": groupID},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> initGroupAttributes({
    required String groupID,
    required Map<String, String> attributes,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "initGroupAttributes",
          buildGroupManagerParam(
            {
              "groupID": groupID,
              "attributes": attributes,
            },
          ),
        ),
      ),
    );
  }

  /* *****-消息模块-******** */
  /*
  *      ┌─┐       ┌─┐
  *   ┌──┘ ┴───────┘ ┴──┐
  *   │                 │
  *   │       ───       │
  *   │   >        <    │
  *   │                 │
  *   │   ...  ⌒  ...   │
  *   │                 │
  *   └───┐         ┌───┘
  *       │         │
  *       │         │
  *       │         │
  *       │         └──────────────┐
  *       │                        │
  *       │                        ├─┐
  *       │                        ┌─┘
  *       │                        │
  *       └─┐  ┐  ┌───────┬──┐  ┌──┘
  *         │ ─┤ ─┤       │ ─┤ ─┤
  *         └──┴──┘       └──┴──┘
  *                神兽保佑，不出bug
  *               只是为了区分不同模块
   */

  /// 创建文本消息
  @override
  Future<V2TimValueCallback<V2TimMsgCreateInfoResult>> createTextMessage({required String text}) async {
    return V2TimValueCallback<V2TimMsgCreateInfoResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "createTextMessage",
          buildMessageMangerParam(
            {
              "text": text,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMsgCreateInfoResult>> createTargetedGroupMessage({required String id, required List<String> receiverList}) async {
    return V2TimValueCallback<V2TimMsgCreateInfoResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "createTargetedGroupMessage",
          buildMessageMangerParam(
            {"id": id, "receiverList": receiverList},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMsgCreateInfoResult>> createCustomMessage({
    required String data,
    String desc = "",
    String extension = "",
  }) async {
    return V2TimValueCallback<V2TimMsgCreateInfoResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "createCustomMessage",
          buildMessageMangerParam(
            {
              "data": data,
              "desc": desc,
              "extension": extension,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMsgCreateInfoResult>> createImageMessage({required String imagePath, dynamic inputElement, String? imageName}) async {
    return V2TimValueCallback<V2TimMsgCreateInfoResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "createImageMessage",
          buildMessageMangerParam(
            {"imagePath": imagePath},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMsgCreateInfoResult>> createSoundMessage({
    required String soundPath,
    required int duration,
  }) async {
    return V2TimValueCallback<V2TimMsgCreateInfoResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "createSoundMessage",
          buildMessageMangerParam(
            {
              "soundPath": soundPath,
              "duration": duration,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMsgCreateInfoResult>> createVideoMessage({
    required String videoFilePath,
    required String type,
    required int duration,
    required String snapshotPath,
    dynamic inputElement,
  }) async {
    return V2TimValueCallback<V2TimMsgCreateInfoResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "createVideoMessage",
          buildMessageMangerParam(
            {
              "videoFilePath": videoFilePath,
              "type": type,
              "duration": duration,
              "snapshotPath": snapshotPath,
            },
          ),
        ),
      ),
    );
  }

  ///发送消息
  @override
  Future<V2TimValueCallback<V2TimMessage>> sendMessage({
    required String id,
    required String receiver,
    required String groupID,
    int priority = 0,
    bool onlineUserOnly = false,
    bool isExcludedFromUnreadCount = false,
    bool isExcludedFromLastMessage = false,
    bool? isSupportMessageExtension = false,
    bool? isExcludedFromContentModeration = false,
    bool needReadReceipt = false,
    Map<String, dynamic>? offlinePushInfo,
    // 自定义消息需要
    String? cloudCustomData, // 云自定义消息字段，只能在消息发送前添加
    String? localCustomData,
  }) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "sendMessage",
          buildMessageMangerParam(
            {
              "id": id,
              "receiver": receiver,
              "groupID": groupID,
              "priority": priority,
              "onlineUserOnly": onlineUserOnly,
              "isExcludedFromUnreadCount": isExcludedFromUnreadCount,
              "isExcludedFromLastMessage": isExcludedFromLastMessage,
              "isSupportMessageExtension": isSupportMessageExtension,
              "isExcludedFromContentModeration": isExcludedFromContentModeration,
              "offlinePushInfo": offlinePushInfo,
              "cloudCustomData": cloudCustomData,
              "localCustomData": localCustomData,
              "needReadReceipt": needReadReceipt,
            },
          ),
        ),
      ),
    );
  }

  /// 发送文件消息
  @override
  Future<V2TimValueCallback<V2TimMsgCreateInfoResult>> createFileMessage({required String filePath, required String fileName, dynamic inputElement}) async {
    return V2TimValueCallback<V2TimMsgCreateInfoResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "createFileMessage",
          buildMessageMangerParam(
            {
              "filePath": filePath,
              "fileName": fileName,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMsgCreateInfoResult>> createTextAtMessage({
    required String text,
    required List<String> atUserList,
  }) async {
    return V2TimValueCallback<V2TimMsgCreateInfoResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'createTextAtMessage',
          buildMessageMangerParam(
            {
              "text": text,
              "atUserList": atUserList,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMsgCreateInfoResult>> createLocationMessage({
    required String desc,
    required double longitude,
    required double latitude,
  }) async {
    return V2TimValueCallback<V2TimMsgCreateInfoResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'createLocationMessage',
          buildMessageMangerParam(
            {"desc": desc, "longitude": longitude, "latitude": latitude},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMsgCreateInfoResult>> createFaceMessage({
    required int index,
    required String data,
  }) async {
    return V2TimValueCallback<V2TimMsgCreateInfoResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'createFaceMessage',
          buildMessageMangerParam(
            {"index": index, "data": data},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMsgCreateInfoResult>> createMergerMessage({
    required List<String> msgIDList,
    required String title,
    required List<String> abstractList,
    required String compatibleText,
    List<String>? webMessageInstanceList,
  }) async {
    return V2TimValueCallback<V2TimMsgCreateInfoResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'createMergerMessage',
          buildMessageMangerParam(
            {
              "msgIDList": msgIDList,
              "title": title,
              "abstractList": abstractList,
              "compatibleText": compatibleText,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMsgCreateInfoResult>> createForwardMessage({required String msgID, String? webMessageInstance}) async {
    return V2TimValueCallback<V2TimMsgCreateInfoResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'createForwardMessage',
          buildMessageMangerParam(
            {"msgID": msgID, "webMessageInstance": webMessageInstance},
          ),
        ),
      ),
    );
  }
  /*
              "receiver": receiver,
              "groupID": groupID,
              "priority": priority,
              "desc": desc,
              "extension": extension,
              "onlineUserOnly": onlineUserOnly,
              "isExcludedFromUnreadCount": isExcludedFromUnreadCount,
              "offlinePushInfo": offlinePushInfo,
  */

  @override
  Future<V2TimValueCallback<V2TimMessage>> sendTextMessage({
    required String text,
    required String receiver,
    required String groupID,
    int priority = 0,
    bool onlineUserOnly = false,
    bool isExcludedFromUnreadCount = false,
    Map<String, dynamic>? offlinePushInfo,
  }) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "sendTextMessage",
          buildMessageMangerParam(
            {
              "text": text,
              "receiver": receiver,
              "groupID": groupID,
              "priority": priority,
              "onlineUserOnly": onlineUserOnly,
              "isExcludedFromUnreadCount": isExcludedFromUnreadCount,
              "offlinePushInfo": offlinePushInfo,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessage>> sendCustomMessage({
    required String data,
    required String receiver,
    required String groupID,
    int priority = 0,
    String desc = "",
    String extension = "",
    bool onlineUserOnly = false,
    bool isExcludedFromUnreadCount = false,
    Map<String, dynamic>? offlinePushInfo,
  }) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "sendCustomMessage",
          buildMessageMangerParam(
            {
              "data": data,
              "receiver": receiver,
              "groupID": groupID,
              "priority": priority,
              "desc": desc,
              "extension": extension,
              "onlineUserOnly": onlineUserOnly,
              "isExcludedFromUnreadCount": isExcludedFromUnreadCount,
              "offlinePushInfo": offlinePushInfo,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessage>> sendImageMessage(
      {required String imagePath,
      required String receiver,
      required String groupID,
      int priority = 0,
      bool onlineUserOnly = false,
      bool isExcludedFromUnreadCount = false,
      Map<String, dynamic>? offlinePushInfo,
      String? fileName,
      Uint8List? fileContent}) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "sendImageMessage",
          buildMessageMangerParam(
            {
              "imagePath": imagePath,
              "receiver": receiver,
              "groupID": groupID,
              "priority": priority,
              "onlineUserOnly": onlineUserOnly,
              "isExcludedFromUnreadCount": isExcludedFromUnreadCount,
              "offlinePushInfo": offlinePushInfo,
              "fileName": fileName,
              "fileContent": fileContent
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessage>> sendVideoMessage({
    required String videoFilePath,
    required String receiver,
    required String type,
    required String snapshotPath,
    required int duration,
    required String groupID,
    int priority = 0,
    bool onlineUserOnly = false,
    bool isExcludedFromUnreadCount = false,
    Map<String, dynamic>? offlinePushInfo,
    String? fileName,
    Uint8List? fileContent,
  }) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "sendVideoMessage",
          buildMessageMangerParam(
            {
              "videoFilePath": videoFilePath,
              "receiver": receiver,
              "snapshotPath": snapshotPath,
              "duration": duration,
              "type": type,
              "groupID": groupID,
              "priority": priority,
              "onlineUserOnly": onlineUserOnly,
              "isExcludedFromUnreadCount": isExcludedFromUnreadCount,
              "offlinePushInfo": offlinePushInfo,
              "fileName": fileName,
              "fileContent": fileContent
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessage>> sendFileMessage(
      {required String filePath,
      required String fileName,
      required String receiver,
      required String groupID,
      int priority = 0,
      bool onlineUserOnly = false,
      bool isExcludedFromUnreadCount = false,
      Map<String, dynamic>? offlinePushInfo,
      Uint8List? fileContent}) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "sendFileMessage",
          buildMessageMangerParam(
            {
              "filePath": filePath,
              "fileName": fileName,
              "receiver": receiver,
              "groupID": groupID,
              "priority": priority,
              "onlineUserOnly": onlineUserOnly,
              "isExcludedFromUnreadCount": isExcludedFromUnreadCount,
              "offlinePushInfo": offlinePushInfo,
              "fileContent": fileContent
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessage>> sendSoundMessage({
    required String soundPath,
    required String receiver,
    required String groupID,
    required int duration,
    int priority = 0,
    bool onlineUserOnly = false,
    bool isExcludedFromUnreadCount = false,
    Map<String, dynamic>? offlinePushInfo,
  }) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "sendSoundMessage",
          buildMessageMangerParam(
            {
              "soundPath": soundPath,
              "receiver": receiver,
              "duration": duration,
              "groupID": groupID,
              "priority": priority,
              "onlineUserOnly": onlineUserOnly,
              "isExcludedFromUnreadCount": isExcludedFromUnreadCount,
              "offlinePushInfo": offlinePushInfo,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessage>> sendTextAtMessage({
    required String text,
    required List<String> atUserList,
    required String receiver,
    required String groupID,
    int priority = 0,
    bool onlineUserOnly = false,
    bool isExcludedFromUnreadCount = false,
    Map<String, dynamic>? offlinePushInfo,
  }) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'sendTextAtMessage',
          buildMessageMangerParam(
            {
              "text": text,
              "atUserList": atUserList,
              "receiver": receiver,
              "groupID": groupID,
              "priority": priority,
              "onlineUserOnly": onlineUserOnly,
              "isExcludedFromUnreadCount": isExcludedFromUnreadCount,
              "offlinePushInfo": offlinePushInfo,
            },
          ),
        ),
      ),
    );
  }

  /// 发送地理位置消息
  ///
  @override
  Future<V2TimValueCallback<V2TimMessage>> sendLocationMessage({
    required String desc,
    required double longitude,
    required double latitude,
    required String receiver,
    required String groupID,
    int priority = 0,
    bool onlineUserOnly = false,
    bool isExcludedFromUnreadCount = false,
    Map<String, dynamic>? offlinePushInfo,
  }) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'sendLocationMessage',
          buildMessageMangerParam(
            {
              "desc": desc,
              "longitude": longitude,
              "latitude": latitude,
              "receiver": receiver,
              "groupID": groupID,
              "priority": priority,
              "onlineUserOnly": onlineUserOnly,
              "isExcludedFromUnreadCount": isExcludedFromUnreadCount,
              "offlinePushInfo": offlinePushInfo,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessage>> sendFaceMessage({
    required int index,
    required String data,
    required String receiver,
    required String groupID,
    int priority = 0,
    bool onlineUserOnly = false,
    bool isExcludedFromUnreadCount = false,
    Map<String, dynamic>? offlinePushInfo,
  }) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'sendFaceMessage',
          buildMessageMangerParam(
            {
              "index": index,
              "data": data,
              "receiver": receiver,
              "groupID": groupID,
              "priority": priority,
              "onlineUserOnly": onlineUserOnly,
              "isExcludedFromUnreadCount": isExcludedFromUnreadCount,
              "offlinePushInfo": offlinePushInfo,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessage>> sendMergerMessage(
      {required List<String> msgIDList,
      required String title,
      required List<String> abstractList,
      required String compatibleText,
      required String receiver,
      required String groupID,
      int priority = 0,
      bool onlineUserOnly = false,
      bool isExcludedFromUnreadCount = false,
      Map<String, dynamic>? offlinePushInfo,
      List<String>? webMessageInstanceList}) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'sendMergerMessage',
          buildMessageMangerParam(
            {
              "msgIDList": msgIDList,
              "title": title,
              "abstractList": abstractList,
              "compatibleText": compatibleText,
              "receiver": receiver,
              "groupID": groupID,
              "priority": priority,
              "onlineUserOnly": onlineUserOnly,
              "isExcludedFromUnreadCount": isExcludedFromUnreadCount,
              "offlinePushInfo": offlinePushInfo,
              "webMessageInstanceList": webMessageInstanceList
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> setLocalCustomData({
    required String msgID,
    required String localCustomData,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "setLocalCustomData",
          buildMessageMangerParam(
            {
              "localCustomData": localCustomData,
              "msgID": msgID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> setLocalCustomInt({
    required String msgID,
    required int localCustomInt,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "setLocalCustomInt",
          buildMessageMangerParam(
            {
              "msgID": msgID,
              "localCustomInt": localCustomInt,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> setCloudCustomData({
    required String data,
    required String msgID,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "setCloudCustomData",
          buildMessageMangerParam(
            {
              "msgID": msgID,
              "data": data,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimMessage>>> getC2CHistoryMessageList({
    required String userID,
    required int count,
    String? lastMsgID,
  }) async {
    return V2TimValueCallback<List<V2TimMessage>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getC2CHistoryMessageList",
          buildMessageMangerParam(
            {
              "userID": userID,
              "count": count,
              "lastMsgID": lastMsgID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimMessage>>> getHistoryMessageList({
    int getType = HistoryMessageGetType.V2TIM_GET_LOCAL_OLDER_MSG,
    String? userID,
    String? groupID,
    int lastMsgSeq = -1,
    required int count,
    String? lastMsgID,
    List<int>? messageTypeList,
    List<int>? messageSeqList,
    int? timeBegin,
    int? timePeriod,
  }) async {
    return V2TimValueCallback<List<V2TimMessage>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getHistoryMessageList",
          buildMessageMangerParam(
            {
              "getType": getType,
              "userID": userID,
              "groupID": groupID,
              'lastMsgSeq': lastMsgSeq,
              "count": count,
              "lastMsgID": lastMsgID,
              "messageTypeList": messageTypeList ?? [],
              "messageSeqList": messageSeqList ?? [],
              "timeBegin": timeBegin,
              "timePeriod": timePeriod,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessageListResult>> getHistoryMessageListV2({
    int getType = HistoryMessageGetType.V2TIM_GET_LOCAL_OLDER_MSG,
    String? userID,
    String? groupID,
    int lastMsgSeq = -1,
    required int count,
    String? lastMsgID,
    List<int>? messageTypeList,
    List<int>? messageSeqList,
    int? timeBegin,
    int? timePeriod,
  }) async {
    return V2TimValueCallback<V2TimMessageListResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getHistoryMessageListV2",
          buildMessageMangerParam(
            {
              "getType": getType,
              "userID": userID,
              "groupID": groupID,
              'lastMsgSeq': lastMsgSeq,
              "count": count,
              "lastMsgID": lastMsgID,
              "messageTypeList": messageTypeList ?? [],
              "messageSeqList": messageSeqList ?? [],
              "timeBegin": timeBegin,
              "timePeriod": timePeriod,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<LinkedHashMap<dynamic, dynamic>> getHistoryMessageListWithoutFormat({
    int getType = HistoryMessageGetType.V2TIM_GET_LOCAL_OLDER_MSG,
    String? userID,
    String? groupID,
    int lastMsgSeq = -1,
    required int count,
    String? lastMsgID,
    List<int>? messageSeqList,
    int? timeBegin,
    int? timePeriod,
  }) async {
    return await _channel.invokeMethod(
      "getHistoryMessageList",
      buildMessageMangerParam(
        {
          "getType": getType,
          "userID": userID,
          "groupID": groupID,
          'lastMsgSeq': lastMsgSeq,
          "count": count,
          "lastMsgID": lastMsgID,
          "messageSeqList": messageSeqList,
          "timeBegin": timeBegin,
          "timePeriod": timePeriod,
        },
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimMessage>>> getGroupHistoryMessageList({
    required String groupID,
    required int count,
    String? lastMsgID,
  }) async {
    return V2TimValueCallback<List<V2TimMessage>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getGroupHistoryMessageList",
          buildMessageMangerParam(
            {
              "groupID": groupID,
              "count": count,
              "lastMsgID": lastMsgID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> revokeMessage({required String msgID, Object? webMessageInstatnce}) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "revokeMessage",
          buildMessageMangerParam(
            {"msgID": msgID, "webMessageInstatnce": webMessageInstatnce},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> markC2CMessageAsRead({
    required String userID,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "markC2CMessageAsRead",
          buildMessageMangerParam(
            {
              "userID": userID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessage>> sendForwardMessage(
      {required String msgID,
      required String receiver,
      required String groupID,
      int priority = 0,
      bool onlineUserOnly = false,
      bool isExcludedFromUnreadCount = false,
      Map<String, dynamic>? offlinePushInfo,
      String? webMessageInstance}) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'sendForwardMessage',
          buildMessageMangerParam(
            {
              "msgID": msgID,
              "receiver": receiver,
              "groupID": groupID,
              "priority": priority,
              "onlineUserOnly": onlineUserOnly,
              "isExcludedFromUnreadCount": isExcludedFromUnreadCount,
              "offlinePushInfo": offlinePushInfo,
              "webMessageInstance": webMessageInstance
            },
          ),
        ),
      ),
    );
  }

  /*
   注意：reSendMessage的onProgress不会返回id
  */
  @override
  Future<V2TimValueCallback<V2TimMessage>> reSendMessage({required String msgID, bool onlineUserOnly = false, Object? webMessageInstatnce}) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "reSendMessage",
          buildMessageMangerParam(
            {"msgID": msgID, "onlineUserOnly": onlineUserOnly, "webMessageInstatnce": webMessageInstatnce},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> setC2CReceiveMessageOpt({
    required List<String> userIDList,
    required int opt,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "setC2CReceiveMessageOpt",
          buildMessageMangerParam(
            {
              "userIDList": userIDList,
              "opt": opt,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimReceiveMessageOptInfo>>> getC2CReceiveMessageOpt({
    required List<String> userIDList,
  }) async {
    return V2TimValueCallback<List<V2TimReceiveMessageOptInfo>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getC2CReceiveMessageOpt",
          buildMessageMangerParam(
            {
              "userIDList": userIDList,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> setGroupReceiveMessageOpt({
    required String groupID,
    required int opt,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "setGroupReceiveMessageOpt",
          buildMessageMangerParam(
            {
              "groupID": groupID,
              "opt": opt,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> markGroupMessageAsRead({
    required String groupID,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "markGroupMessageAsRead",
          buildMessageMangerParam(
            {
              "groupID": groupID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> deleteMessageFromLocalStorage({
    required String msgID,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "deleteMessageFromLocalStorage",
          buildMessageMangerParam(
            {
              "msgID": msgID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> deleteMessages({required List<String> msgIDs, List<dynamic>? webMessageInstanceList}) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "deleteMessages",
          buildMessageMangerParam(
            {
              "msgIDs": msgIDs,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessage>> insertGroupMessageToLocalStorage({
    required String data,
    required String groupID,
    required String sender,
  }) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "insertGroupMessageToLocalStorage",
          buildMessageMangerParam(
            {
              "data": data,
              "groupID": groupID,
              "sender": sender,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessage>> insertC2CMessageToLocalStorage({
    required String data,
    required String userID,
    required String sender,
  }) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "insertC2CMessageToLocalStorage",
          buildMessageMangerParam(
            {
              "data": data,
              "userID": userID,
              "sender": sender,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> clearC2CHistoryMessage({
    required String userID,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "clearC2CHistoryMessage",
          buildMessageMangerParam(
            {
              "userID": userID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> clearGroupHistoryMessage({
    required String groupID,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "clearGroupHistoryMessage",
          buildMessageMangerParam(
            {
              "groupID": groupID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessageSearchResult>> searchLocalMessages({
    required V2TimMessageSearchParam searchParam,
  }) async {
    return V2TimValueCallback<V2TimMessageSearchResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "searchLocalMessages",
          buildMessageMangerParam(
            {
              "searchParam": searchParam.toJson(),
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimMessage>>> findMessages({
    required List<String> messageIDList,
  }) async {
    return V2TimValueCallback<List<V2TimMessage>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "findMessages",
          buildMessageMangerParam(
            {
              "messageIDList": messageIDList,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> markAllMessageAsRead() async {
    return V2TimCallback.fromJson(formatJson(await _channel.invokeMethod("markAllMessageAsRead", buildMessageMangerParam({}))));
  }

  @override

  /// 添加高级消息的事件监听器
  ///
  Future<void> addAdvancedMsgListener({
    required V2TimAdvancedMsgListener listener,
  }) async {
    final String uuid = Utils.generateUniqueString();
    advancedMsgListenerList[uuid] = listener;
    return await _channel.invokeMethod("addAdvancedMsgListener", buildMessageMangerParam({"listenerUuid": uuid}));
  }

  @override

  /// 移除高级消息监听器
  ///
  Future<void> removeAdvancedMsgListener({
    V2TimAdvancedMsgListener? listener,
  }) async {
    var listenerUuid = "";
    if (listener != null) {
      listenerUuid = advancedMsgListenerList.keys.firstWhere((k) => advancedMsgListenerList[k] == listener, orElse: () => "");
      advancedMsgListenerList.remove(listenerUuid);
    } else {
      advancedMsgListenerList.clear();
    }
    return _channel.invokeMethod("removeAdvancedMsgListener", buildMessageMangerParam({"listenerUuid": listenerUuid}));
  }

  @override
  Future<V2TimCallback> sendMessageReadReceipts({
    required List<String> messageIDList,
  }) async {
    return V2TimCallback.fromJson(formatJson(await _channel.invokeMethod(
      "sendMessageReadReceipts",
      buildMessageMangerParam(
        {
          "messageIDList": messageIDList,
        },
      ),
    )));
  }

  @override
  Future<V2TimValueCallback<List<V2TimMessageReceipt>>> getMessageReadReceipts({
    required List<String> messageIDList,
  }) async {
    return V2TimValueCallback<List<V2TimMessageReceipt>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getMessageReadReceipts",
          buildMessageMangerParam(
            {
              "messageIDList": messageIDList,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimGroupMessageReadMemberList>> getGroupMessageReadMemberList({
    required String messageID,
    required GetGroupMessageReadMemberListFilter filter,
    int nextSeq = 0,
    int count = 100,
  }) async {
    return V2TimValueCallback<V2TimGroupMessageReadMemberList>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getGroupMessageReadMemberList",
          buildMessageMangerParam(
            {
              "messageID": messageID,
              "filter": filter.index,
              "nextSeq": nextSeq,
              "count": count,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimGroupInfo>>> getJoinedCommunityList() async {
    return V2TimValueCallback<List<V2TimGroupInfo>>.fromJson(formatJson(await _channel.invokeMethod('getJoinedCommunityList', buildGroupManagerParam({}))));
  }

  @override
  Future<V2TimValueCallback<String>> createTopicInCommunity({
    required String groupID,
    required V2TimTopicInfo topicInfo,
  }) async {
    return V2TimValueCallback<String>.fromJson(formatJson(await _channel.invokeMethod('createTopicInCommunity', buildGroupManagerParam({"groupID": groupID, "topicInfo": topicInfo.toJson()}))));
  }

  @override
  Future<V2TimValueCallback<List<V2TimTopicOperationResult>>> deleteTopicFromCommunity({
    required String groupID,
    required List<String> topicIDList,
  }) async {
    return V2TimValueCallback<List<V2TimTopicOperationResult>>.fromJson(formatJson(await _channel.invokeMethod('deleteTopicFromCommunity', buildGroupManagerParam({"groupID": groupID, "topicIDList": topicIDList}))));
  }

  @override
  Future<V2TimCallback> setTopicInfo({
    required String groupID,
    required V2TimTopicInfo topicInfo,
  }) async {
    return V2TimCallback.fromJson(formatJson(await _channel.invokeMethod(
        'setTopicInfo',
        buildGroupManagerParam({
          "topicInfo": topicInfo.toJson(),
          "groupID": groupID,
        }))));
  }

  @override
  Future<V2TimValueCallback<List<V2TimTopicInfoResult>>> getTopicInfoList({
    required String groupID,
    required List<String> topicIDList,
  }) async {
    return V2TimValueCallback<List<V2TimTopicInfoResult>>.fromJson(formatJson(await _channel.invokeMethod('getTopicInfoList', buildGroupManagerParam({"groupID": groupID, "topicIDList": topicIDList}))));
  }

  @override
  Future<V2TimValueCallback<V2TimMessageChangeInfo>> modifyMessage({
    required V2TimMessage message,
  }) async {
    return V2TimValueCallback<V2TimMessageChangeInfo>.fromJson(formatJson(await _channel.invokeMethod(
        'modifyMessage',
        buildMessageMangerParam({
          "message": message.toJson(),
        }))));
  }

  @override
  Future<V2TimValueCallback<V2TimMessage>> appendMessage({
    required String createMessageBaseId,
    required String createMessageAppendId,
  }) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(formatJson(await _channel.invokeMethod(
        'appendMessage',
        buildMessageMangerParam({
          "createMessageBaseId": createMessageBaseId,
          "createMessageAppendId": createMessageAppendId,
        }))));
  }

  @override
  Future<V2TimValueCallback<List<V2TimUserStatus>>> getUserStatus({
    required List<String> userIDList,
  }) async {
    return V2TimValueCallback<List<V2TimUserStatus>>.fromJson(formatJson(await _channel.invokeMethod(
        'getUserStatus',
        buildTimManagerParam({
          "userIDList": userIDList,
        }))));
  }

  @override
  Future<V2TimCallback> setSelfStatus({
    required String status,
  }) async {
    return V2TimCallback.fromJson(formatJson(await _channel.invokeMethod(
        'setSelfStatus',
        buildTimManagerParam({
          "status": status,
        }))));
  }

  @override
  Future<V2TimValueCallback<int>> checkAbility() async {
    return V2TimValueCallback<int>.fromJson(formatJson(await _channel.invokeMethod(
      'checkAbility',
      buildTimManagerParam(
        {},
      ),
    )));
  }

  @override
  Future<V2TimCallback> subscribeUserStatus({
    required List<String> userIDList,
  }) async {
    return V2TimCallback.fromJson(formatJson(await _channel.invokeMethod(
      'subscribeUserStatus',
      buildTimManagerParam(
        {
          "userIDList": userIDList,
        },
      ),
    )));
  }

  @override
  Future<V2TimCallback> unsubscribeUserStatus({
    required List<String> userIDList,
  }) async {
    return V2TimCallback.fromJson(formatJson(await _channel.invokeMethod(
      'unsubscribeUserStatus',
      buildTimManagerParam(
        {
          "userIDList": userIDList,
        },
      ),
    )));
  }

  @override
  Future<V2TimValueCallback<List<V2TimConversationOperationResult>>> setConversationCustomData({
    required String customData,
    required List<String> conversationIDList,
  }) async {
    return V2TimValueCallback<List<V2TimConversationOperationResult>>.fromJson(formatJson(await _channel.invokeMethod(
      'setConversationCustomData',
      buildConversationManagerParam(
        {
          "customData": customData,
          "conversationIDList": conversationIDList,
        },
      ),
    )));
  }

  @override
  Future<V2TimValueCallback<V2TimConversationResult>> getConversationListByFilter({
    required V2TimConversationFilter filter,
    required int nextSeq,
    required int count,
  }) async {
    return V2TimValueCallback<V2TimConversationResult>.fromJson(formatJson(await _channel.invokeMethod(
      'getConversationListByFilter',
      buildConversationManagerParam(
        {
          "filter": filter.toJson(),
          "nextSeq": nextSeq,
          "count": count,
        },
      ),
    )));
  }

  @override
  Future<V2TimValueCallback<List<V2TimConversationOperationResult>>> markConversation({
    required int markType,
    required bool enableMark,
    required List<String> conversationIDList,
  }) async {
    return V2TimValueCallback<List<V2TimConversationOperationResult>>.fromJson(formatJson(await _channel.invokeMethod(
      'markConversation',
      buildConversationManagerParam(
        {
          "markType": markType,
          "enableMark": enableMark,
          "conversationIDList": conversationIDList,
        },
      ),
    )));
  }

  @override
  Future<V2TimValueCallback<List<V2TimConversationOperationResult>>> createConversationGroup({
    required String groupName,
    required List<String> conversationIDList,
  }) async {
    return V2TimValueCallback<List<V2TimConversationOperationResult>>.fromJson(formatJson(await _channel.invokeMethod(
      'createConversationGroup',
      buildConversationManagerParam(
        {
          "groupName": groupName,
          "conversationIDList": conversationIDList,
        },
      ),
    )));
  }

  @override
  Future<V2TimValueCallback<List<String>>> getConversationGroupList() async {
    return V2TimValueCallback<List<String>>.fromJson(formatJson(await _channel.invokeMethod(
      'getConversationGroupList',
      buildConversationManagerParam(
        {},
      ),
    )));
  }

  @override
  Future<V2TimCallback> deleteConversationGroup({
    required String groupName,
  }) async {
    return V2TimCallback.fromJson(formatJson(await _channel.invokeMethod(
      'deleteConversationGroup',
      buildConversationManagerParam(
        {
          "groupName": groupName,
        },
      ),
    )));
  }

  @override
  Future<V2TimCallback> renameConversationGroup({
    required String oldName,
    required String newName,
  }) async {
    return V2TimCallback.fromJson(formatJson(await _channel.invokeMethod(
      'renameConversationGroup',
      buildConversationManagerParam(
        {
          "oldName": oldName,
          "newName": newName,
        },
      ),
    )));
  }

  @override
  Future<V2TimValueCallback<List<V2TimMessage>>> downloadMergerMessage({
    required String msgID,
  }) async {
    return V2TimValueCallback<List<V2TimMessage>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'downloadMergerMessage',
          buildMessageMangerParam(
            {
              "msgID": msgID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimConversationOperationResult>>> addConversationsToGroup({
    required String groupName,
    required List<String> conversationIDList,
  }) async {
    return V2TimValueCallback<List<V2TimConversationOperationResult>>.fromJson(formatJson(await _channel.invokeMethod(
      'addConversationsToGroup',
      buildConversationManagerParam(
        {
          "groupName": groupName,
          "conversationIDList": conversationIDList,
        },
      ),
    )));
  }

  @override
  Future<V2TimValueCallback<List<V2TimConversationOperationResult>>> deleteConversationList({
    required List<String> conversationIDList,
    required bool clearMessage,
  }) async {
    return V2TimValueCallback<List<V2TimConversationOperationResult>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'deleteConversationList',
          buildConversationManagerParam(
            {
              "conversationIDList": conversationIDList,
              "clearMessage": clearMessage,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> subscribeUnreadMessageCountByFilter({
    required V2TimConversationFilter filter,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'subscribeUnreadMessageCountByFilter',
          buildConversationManagerParam(
            {
              "filter": filter.toJson(),
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> unsubscribeUnreadMessageCountByFilter({
    required V2TimConversationFilter filter,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'unsubscribeUnreadMessageCountByFilter',
          buildConversationManagerParam(
            {
              "filter": filter.toJson(),
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> cleanConversationUnreadMessageCount({
    required String conversationID,
    required int cleanTimestamp,
    required int cleanSequence,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'cleanConversationUnreadMessageCount',
          buildConversationManagerParam(
            {
              "conversationID": conversationID,
              "cleanTimestamp": cleanTimestamp,
              "cleanSequence": cleanSequence,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<int>> getUnreadMessageCountByFilter({
    required V2TimConversationFilter filter,
  }) async {
    return V2TimValueCallback<int>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'getUnreadMessageCountByFilter',
          buildConversationManagerParam(
            {
              "filter": filter.toJson(),
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<String>> convertVoiceToText({
    required String msgID,
    required String language, // "zh (cmn-Hans-CN)" "yue-Hant-HK" "en-US" "ja-JP"
  }) async {
    return V2TimValueCallback<String>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          'convertVoiceToText',
          buildMessageMangerParam(
            {
              "msgID": msgID,
              "language": language,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimConversationOperationResult>>> deleteConversationsFromGroup({
    required String groupName,
    required List<String> conversationIDList,
  }) async {
    return V2TimValueCallback<List<V2TimConversationOperationResult>>.fromJson(formatJson(await _channel.invokeMethod(
      'deleteConversationsFromGroup',
      buildConversationManagerParam(
        {
          "groupName": groupName,
          "conversationIDList": conversationIDList,
        },
      ),
    )));
  }

  // 信令
  @override
  Future<void> addSignalingListener({
    required V2TimSignalingListener listener,
  }) async {
    final String listenerUuid = Utils.generateUniqueString();
    signalingListenerList[listenerUuid] = listener;
    return _channel.invokeMethod(
      "addSignalingListener",
      buildSignalingManagerParam(
        {
          "listenerUuid": listenerUuid,
        },
      ),
    );
  }

  @override
  Future<void> removeSignalingListener({
    V2TimSignalingListener? listener,
  }) async {
    var listenerUuid = "";
    if (listener != null) {
      listenerUuid = signalingListenerList.keys.firstWhere((k) => signalingListenerList[k] == listener, orElse: () => "");
      signalingListenerList.remove(listenerUuid);
    } else {
      signalingListenerList.clear();
    }
    return _channel.invokeMethod(
      "removeSignalingListener",
      buildSignalingManagerParam(
        {
          "listenerUuid": listenerUuid,
        },
      ),
    );
  }

  @override
  Future<V2TimValueCallback<String>> invite({
    required String invitee,
    required String data,
    int timeout = 30,
    bool onlineUserOnly = false,
    OfflinePushInfo? offlinePushInfo,
  }) async {
    return V2TimValueCallback<String>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "invite",
          buildSignalingManagerParam(
            {
              "invitee": invitee,
              "data": data,
              "timeout": timeout,
              "onlineUserOnly": onlineUserOnly,
              "offlinePushInfo": offlinePushInfo?.toJson(),
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<String>> inviteInGroup({
    required String groupID,
    required List<String> inviteeList,
    required String data,
    int timeout = 30,
    bool onlineUserOnly = false,
  }) async {
    return V2TimValueCallback<String>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "inviteInGroup",
          buildSignalingManagerParam(
            {"groupID": groupID, "inviteeList": inviteeList, "data": data, "timeout": timeout, "onlineUserOnly": onlineUserOnly},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> cancel({
    required String inviteID,
    String? data,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "cancel",
          buildSignalingManagerParam(
            {
              "inviteID": inviteID,
              "data": data,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> accept({
    required String inviteID,
    String? data,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "accept",
          buildSignalingManagerParam(
            {
              "inviteID": inviteID,
              "data": data,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> reject({
    required String inviteID,
    String? data,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "reject",
          buildSignalingManagerParam(
            {
              "inviteID": inviteID,
              "data": data,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimSignalingInfo>> getSignalingInfo({
    required String msgID,
  }) async {
    return V2TimValueCallback<V2TimSignalingInfo>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getSignalingInfo",
          buildSignalingManagerParam(
            {
              "msgID": msgID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> addInvitedSignaling({
    required V2TimSignalingInfo info,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "addInvitedSignaling",
          buildSignalingManagerParam(
            {
              "info": info.toJson(),
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimMessageExtensionResult>>> setMessageExtensions({
    required String msgID,
    required List<V2TimMessageExtension> extensions,
  }) async {
    return V2TimValueCallback<List<V2TimMessageExtensionResult>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "setMessageExtensions",
          buildMessageMangerParam(
            {
              "msgID": msgID,
              "extensions": extensions
                  .map(
                    (e) => Map.from(
                      {
                        "key": e.extensionKey,
                        "value": e.extensionValue,
                      },
                    ),
                  )
                  .toList(),
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimMessageExtension>>> getMessageExtensions({
    required String msgID,
  }) async {
    return V2TimValueCallback<List<V2TimMessageExtension>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getMessageExtensions",
          buildMessageMangerParam(
            {
              "msgID": msgID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<V2TimMessageExtensionResult>>> deleteMessageExtensions({
    required String msgID,
    required List<String> keys,
  }) async {
    return V2TimValueCallback<List<V2TimMessageExtensionResult>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "deleteMessageExtensions",
          buildMessageMangerParam(
            {
              "msgID": msgID,
              "keys": keys,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessageOnlineUrl>> getMessageOnlineUrl({
    required String msgID,
  }) async {
    return V2TimValueCallback<V2TimMessageOnlineUrl>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getMessageOnlineUrl",
          buildMessageMangerParam(
            {
              "msgID": msgID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> downloadMessage({
    required String msgID,
    required int messageType,
    required int imageType, // 图片类型，仅messageType为图片消息是有效
    required bool isSnapshot, // 是否是视频封面，仅messageType为视频消息是有效
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "downloadMessage",
          buildMessageMangerParam(
            {
              "msgID": msgID,
              "messageType": messageType,
              "imageType": imageType,
              "isSnapshot": isSnapshot,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<Map<String, int>>> setGroupCounters({
    required String groupID,
    required Map<String, int> counters,
  }) async {
    return V2TimValueCallback<Map<String, int>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "setGroupCounters",
          buildGroupManagerParam(
            {
              "groupID": groupID,
              "counters": counters,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<Map<String, int>>> getGroupCounters({
    required String groupID,
    required List<String> keys,
  }) async {
    return V2TimValueCallback<Map<String, int>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getGroupCounters",
          buildGroupManagerParam(
            {
              "groupID": groupID,
              "keys": keys,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<Map<String, int>>> increaseGroupCounter({
    required String groupID,
    required String key,
    required int value,
  }) async {
    return V2TimValueCallback<Map<String, int>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "increaseGroupCounter",
          buildGroupManagerParam(
            {
              "groupID": groupID,
              "key": key,
              "value": value,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<Map<String, int>>> decreaseGroupCounter({
    required String groupID,
    required String key,
    required int value,
  }) async {
    return V2TimValueCallback<Map<String, int>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "decreaseGroupCounter",
          buildGroupManagerParam(
            {
              "groupID": groupID,
              "key": key,
              "value": value,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<Map<String, String>>> translateText({
    required List<String> texts,
    required String targetLanguage,
    String? sourceLanguage,
  }) async {
    return V2TimValueCallback<Map<String, String>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "translateText",
          buildMessageMangerParam(
            {
              "texts": texts,
              "targetLanguage": targetLanguage,
              "sourceLanguage": sourceLanguage ?? "auto",
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<List<String>>> setAvChatRoomCanFindMessage({
    required List<String> avchatroomIDs,
    int eachGroupMessageNums = 20,
  }) async {
    return V2TimValueCallback<List<String>>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "setAvChatRoomCanFindMessage",
          buildMessageMangerParam(
            {
              "avchatroomIDs": avchatroomIDs,
              "eachGroupMessageNums": eachGroupMessageNums,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> subscribeUserInfo({
    required List<String> userIDList,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "subscribeUserInfo",
          buildTimManagerParam(
            {
              "userIDList": userIDList,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> unsubscribeUserInfo({
    required List<String> userIDList,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "unsubscribeUserInfo",
          buildTimManagerParam(
            {
              "userIDList": userIDList,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> markGroupMemberList({
    required String groupID,
    required List<String> memberIDList,
    required int markType,
    required bool enableMark,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "markGroupMemberList",
          buildGroupManagerParam(
            {
              "groupID": groupID,
              "memberIDList": memberIDList,
              "markType": markType,
              "enableMark": enableMark,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> setAllReceiveMessageOpt({
    required int opt,
    required int startHour,
    required int startMinute,
    required int startSecond,
    required int duration,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "setAllReceiveMessageOpt",
          buildMessageMangerParam(
            {
              "opt": opt,
              "startHour": startHour,
              "startMinute": startMinute,
              "startSecond": startSecond,
              "duration": duration,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> setAllReceiveMessageOptWithTimestamp({
    required int opt,
    required int startTimeStamp,
    required int duration,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "setAllReceiveMessageOptWithTimestamp",
          buildMessageMangerParam(
            {
              "opt": opt,
              "startTimeStamp": startTimeStamp,
              "duration": duration,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimReceiveMessageOptInfo>> getAllReceiveMessageOpt() async {
    return V2TimValueCallback<V2TimReceiveMessageOptInfo>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getAllReceiveMessageOpt",
          buildMessageMangerParam(
            {},
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> addMessageReaction({
    required String msgID,
    required String reactionID,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "addMessageReaction",
          buildMessageMangerParam(
            {
              "msgID": msgID,
              "reactionID": reactionID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> removeMessageReaction({
    required String msgID,
    required String reactionID,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "removeMessageReaction",
          buildMessageMangerParam(
            {
              "msgID": msgID,
              "reactionID": reactionID,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessageReactionResult>> getMessageReactions({
    required List<String> msgIDList,
    required int maxUserCountPerReaction,
  }) async {
    return V2TimValueCallback<V2TimMessageReactionResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getMessageReactions",
          buildMessageMangerParam(
            {
              "msgIDList": msgIDList,
              "maxUserCountPerReaction": maxUserCountPerReaction,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessageReactionUserResult>> getAllUserListOfMessageReaction({
    required String msgID,
    required String reactionID,
    required int nextSeq,
    required int count,
  }) async {
    return V2TimValueCallback<V2TimMessageReactionUserResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "getAllUserListOfMessageReaction",
          buildMessageMangerParam(
            {
              "msgID": msgID,
              "reactionID": reactionID,
              "nextSeq": nextSeq,
              "count": count,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<void> uikitTrace({
    required String trace,
  }) async {
    await _channel.invokeMethod(
      "uikitTrace",
      buildTimManagerParam(
        {
          "trace": trace,
        },
      ),
    );
  }

  @override
  Future<V2TimValueCallback<V2TimMessageSearchResult>> searchCloudMessages({
    required V2TimMessageSearchParam searchParam,
  }) async {
    return V2TimValueCallback<V2TimMessageSearchResult>.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "searchCloudMessages",
          buildMessageMangerParam(
            {
              "searchParam": searchParam.toJson(),
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> setOfflinePushConfig({
    required double businessID,
    required String token,
    bool isTPNSToken = false,
    bool isVoip = false,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          Platform.isIOS ? (isVoip ? "setVOIP" : "setAPNS") : "setOfflinePushConfig",
          buildOfflinePushParam(
            {
              "ability": Utils.getAbility(),
              "businessID": businessID,
              "token": token,
              "isTPNSToken": isTPNSToken,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> doBackground({
    required int unreadCount,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "doBackground",
          buildOfflinePushParam(
            {
              "ability": Utils.getAbility(),
              "unreadCount": unreadCount,
            },
          ),
        ),
      ),
    );
  }

  @override
  Future<V2TimCallback> doForeground() async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "doForeground",
          buildOfflinePushParam(
            {
              "ability": Utils.getAbility(),
            },
          ),
        ),
      ),
    );
  }

  Map buildGroupManagerParam(Map param) {
    param["TIMManagerName"] = "groupManager";
    try {
      param["ability"] = Utils.getAbility();
      // String stacktrace = StackTrace.current.toString();
      // param["stacktrace"] = stacktrace;
    } catch (err) {
      // err
    }
    return param;
  }

  Map buildSignalingManagerParam(Map param) {
    param["TIMManagerName"] = "signalingManager";
    try {
      param["ability"] = Utils.getAbility();
      // String stacktrace = StackTrace.current.toString();
      // param["stacktrace"] = stacktrace;
    } catch (err) {
      // err
    }
    return param;
  }

  Map buildFriendManagerParam(Map param) {
    param["TIMManagerName"] = "friendshipManager";
    // String stacktrace = StackTrace.current.toString();
    // param["stacktrace"] = stacktrace;
    try {
      param["ability"] = Utils.getAbility();
    } catch (err) {
      // err
    }
    return param;
  }

  Map buildOfflinePushParam(Map param) {
    param["TIMManagerName"] = "offlinePushManager";
    return param;
  }

  Map buildTimManagerParam(Map param) {
    param["TIMManagerName"] = "timManager";
    // String stacktrace = StackTrace.current.toString();
    // param["stacktrace"] = stacktrace;
    try {
      param["ability"] = Utils.getAbility();
    } catch (err) {
      // err
    }
    return param;
  }

  Map buildMessageMangerParam(Map param) {
    param["TIMManagerName"] = "messageManager";
    try {
      param["ability"] = Utils.getAbility();
      // String stacktrace = StackTrace.current.toString();
      // param["stacktrace"] = stacktrace;
    } catch (err) {
      // err
    }
    return param;
  }

  ///@nodoc
  Map buildConversationManagerParam(Map param) {
    param["TIMManagerName"] = "conversationManager";
    // String stacktrace = StackTrace.current.toString();
    // param["stacktrace"] = stacktrace;
    try {
      param["ability"] = Utils.getAbility();
    } catch (err) {
      // err
    }
    return param;
  }

  formatJson(jsonSrc) {
    return json.decode(json.encode(jsonSrc));
  }
}
