// Copyright (c) 2023 Tencent. All rights reserved.

#ifndef __V2TIM_COMMUNITY_H__
#define __V2TIM_COMMUNITY_H__

#include "V2TIMCommon.h"
#include "V2TIMDefine.h"
#include "V2TIMGroup.h"

// 话题资料修改标记位
enum V2TIMTopicInfoModifyFlag {
    // 未定义
    V2TIM_COMMUNITY_MODIFY_FLAG_UNKNOWN = 0x00,
    // 名称
    V2TIM_COMMUNITY_MODIFY_FLAG_TOPIC_NAME = 0x01,
    // 公告
    V2TIM_COMMUNITY_MODIFY_FLAG_TOPIC_NOTIFICATION = 0x01 << 1,
    // 简介
    V2TIM_COMMUNITY_MODIFY_FLAG_TOPIC_INTRODUCTION = 0x01 << 2,
    // 头像
    V2TIM_COMMUNITY_MODIFY_FLAG_TOPIC_FACE_URL = 0x01 << 3,
    // 禁言
    V2TIM_COMMUNITY_MODIFY_FLAG_TOPIC_MUTE_ALL = 0x01 << 8,
    // 话题自定义字段
    V2TIM_COMMUNITY_MODIFY_FLAG_TOPIC_CUSTOM_STRING = 0x1 << 11,
    // 话题草稿
    V2TIM_COMMUNITY_MODIFY_FLAG_TOPIC_DRAFT = 0x1 << 14,
};

/////////////////////////////////////////////////////////////////////////////////
//
//             话题基本资料
//
/////////////////////////////////////////////////////////////////////////////////

struct V2TIMMessage;

struct TIM_API V2TIMTopicInfo {
    /// 话题 ID，只能在创建话题或者修改话题信息的时候设置。组成方式为：社群 ID + @TOPIC#_xxx，例如社群 ID 为 @TGS#_123，则话题 ID 为 @TGS#_123@TOPIC#_xxx
    V2TIMString topicID;
    /// 话题名称，最长 150 字节，使用 UTF-8 编码
    V2TIMString topicName;
    /// 话题头像，最长 500 字节，使用 UTF-8 编码
    V2TIMString topicFaceURL;
    /// 话题介绍，最长 400 字节，使用 UTF-8 编码
    V2TIMString introduction;
    /// 话题公告，最长 400 字节，使用 UTF-8 编码
    V2TIMString notification;
    /// 话题全员禁言
    bool isAllMuted;
    /// 当前用户在话题中的禁言时间
    uint32_t selfMuteTime;
    /// 话题自定义字段
    V2TIMString customString;
    /// 话题消息接收选项，修改话题消息接收选项请调用 setGroupReceiveMessageOpt 接口
    V2TIMReceiveMessageOpt recvOpt;
    /// 话题草稿
    V2TIMString draftText;
    /// 话题消息未读数量
    uint64_t unreadCount;
    /// 话题 lastMessage
    V2TIMMessage *lastMessage;
    /// 话题 at 信息列表
    V2TIMGroupAtInfoVector groupAtInfoList;
    /// 话题创建时间，单位：秒
    uint32_t createTime;
    /// 话题资料修改标记位
    /// 枚举 V2TIMGroupInfoModifyFlag 列出哪些字段支持修改，如果您修改群资料，请设置这个字段值
    /// 如果您同时修改多个字段，多个枚举值按位或 | 组合，例如，同时修改群名称和头像
    /// info.topicName = "new group name";
    /// info.topicFaceURL = "new face url";
    /// info.modifyFlag = V2TIM_GROUP_INFO_MODIFY_FLAG_GROUP_NAME |
    /// V2TIM_GROUP_INFO_MODIFY_FLAG_FACE_URL;
    uint32_t modifyFlag;

    V2TIMTopicInfo();
    V2TIMTopicInfo(const V2TIMTopicInfo& topicInfo);
    V2TIMTopicInfo& operator =(const V2TIMTopicInfo& topicInfo);
    ~V2TIMTopicInfo();
};

/////////////////////////////////////////////////////////////////////////////////
//
//             V2TIMTopicOperationResult
//
/////////////////////////////////////////////////////////////////////////////////

struct TIM_API V2TIMTopicOperationResult {
    /// 结果 0：成功；非0：失败
    int32_t errorCode;
    /// 如果删除失败，会返回错误信息
    V2TIMString errorMsg;
    /// 如果删除成功，会返回对应的 topicID
    V2TIMString topicID;

    V2TIMTopicOperationResult();
    V2TIMTopicOperationResult(const V2TIMTopicOperationResult& result);
    ~V2TIMTopicOperationResult();
};

DEFINE_VECTOR(V2TIMTopicOperationResult)
typedef TXV2TIMTopicOperationResultVector V2TIMTopicOperationResultVector;

/////////////////////////////////////////////////////////////////////////////////
//
//             V2TIMTopicInfoResult
//
/////////////////////////////////////////////////////////////////////////////////

struct TIM_API V2TIMTopicInfoResult {
    /// 结果 0：成功；非0：失败
    int32_t errorCode;
    /// 如果获取失败，会返回错误信息
    V2TIMString errorMsg;
    /// 如果获取成功，会返回对应的 info
    V2TIMTopicInfo topicInfo;

    V2TIMTopicInfoResult();
    V2TIMTopicInfoResult(const V2TIMTopicInfoResult& result);
    ~V2TIMTopicInfoResult();
};

DEFINE_VECTOR(V2TIMTopicInfoResult)
typedef TXV2TIMTopicInfoResultVector V2TIMTopicInfoResultVector;


#endif  // __V2TIM_COMMUNITY_H__
